# Copyright 2017 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v3 import versions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestVersionsClient(base.BaseServiceTest):

    FAKE_VERSIONS_INFO = {
        "versions": [
            {
                "status": "DEPRECATED", "updated": "2016-05-02T20:25:19Z",
                "links": [
                    {"href": "http://docs.openstack.org/", "type": "text/html",
                     "rel": "describedby"},
                    {"href": "https://10.30.197.39:8776/v1/", "rel": "self"}
                ],
                "min_version": "",
                "version": "",
                "media-types": [
                    {"base": "application/json",
                     "type": "application/vnd.openstack.volume+json;version=1"}
                ],
                "id": "v1.0"
            },
            {
                "status": "DEPRECATED", "updated": "2017-02-25T12:00:00Z",
                "links": [
                    {"href": "http://docs.openstack.org/", "type": "text/html",
                     "rel": "describedby"},
                    {"href": "https://10.30.197.39:8776/v2/", "rel": "self"}
                ],
                "min_version": "",
                "version": "",
                "media-types": [
                    {"base": "application/json",
                     "type": "application/vnd.openstack.volume+json;version=1"}
                ],
                "id": "v2.0"
            },
            {
                "status": "CURRENT", "updated": "2016-02-08T12:20:21Z",
                "links": [
                    {"href": "http://docs.openstack.org/", "type": "text/html",
                     "rel": "describedby"},
                    {"href": "https://10.30.197.39:8776/v3/", "rel": "self"}
                ],
                "min_version": "3.0",
                "version": "3.28",
                "media-types": [
                    {"base": "application/json",
                     "type": "application/vnd.openstack.volume+json;version=1"}
                ],
                "id": "v3.0"
            }
        ]
    }

    FAKE_VERSION_DETAILS = {
        "versions": [
            {
                "id": "v3.0",
                "links": [
                    {"href": "https://docs.openstack.org/",
                     "type": "text/html", "rel": "describedby"},
                    {"href": "http://127.0.0.1:44895/v3/", "rel": "self"}
                ],
                "media-types": [
                    {"base": "application/json",
                     "type": "application/vnd.openstack.volume+json;version=3"}
                ],
                "min_version": "3.0",
                "status": "CURRENT",
                "updated": "2018-07-17T00:00:00Z",
                "version": "3.59"
            }
        ]
    }

    def setUp(self):
        super(TestVersionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = versions_client.VersionsClient(fake_auth,
                                                     'volume',
                                                     'regionOne')

    def _test_get_base_version_url(self, url, expected_base_url):
        fake_auth = fake_auth_provider.FakeAuthProvider(fake_base_url=url)
        client = versions_client.VersionsClient(fake_auth,
                                                'volume',
                                                'regionOne')
        self.assertEqual(expected_base_url,
                         client._get_base_version_url())

    def _test_list_versions(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_versions,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            self.FAKE_VERSIONS_INFO,
            bytes_body,
            300)

    def _test_show_version(self, version, bytes_body=False):
        self.check_service_client_function(
            self.client.show_version,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            self.FAKE_VERSION_DETAILS,
            bytes_body,
            200, version=version)

    def test_list_versions_with_str_body(self):
        self._test_list_versions()

    def test_list_versions_with_bytes_body(self):
        self._test_list_versions(bytes_body=True)

    def test_show_version_details_with_str_body(self):
        self._test_show_version('v3')

    def test_show_version_details_with_bytes_body(self):
        self._test_show_version('v3', bytes_body=True)

    def test_get_base_version_url_app_name(self):
        self._test_get_base_version_url('https://bar.org/volume/v1/123',
                                        'https://bar.org/volume/')
        self._test_get_base_version_url('https://bar.org/volume/v2/123',
                                        'https://bar.org/volume/')
        self._test_get_base_version_url('https://bar.org/volume/v3/123',
                                        'https://bar.org/volume/')



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import tempest_fixtures as fixtures
from tempest.lib import decorators


class HostsAdminTestJSON(base.BaseV2ComputeAdminTest):
    """Tests nova hosts API using admin privileges."""

    max_microversion = '2.42'

    @classmethod
    def setup_clients(cls):
        super(HostsAdminTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.hosts_client

    @decorators.idempotent_id('9bfaf98d-e2cb-44b0-a07e-2558b2821e4f')
    def test_list_hosts(self):
        """Listing nova hosts"""
        hosts = self.client.list_hosts()['hosts']
        self.assertGreaterEqual(len(hosts), 2, str(hosts))

    @decorators.idempotent_id('5dc06f5b-d887-47a2-bb2a-67762ef3c6de')
    def test_list_hosts_with_zone(self):
        """Listing nova hosts with specified availability zone"""
        self.useFixture(fixtures.LockFixture('availability_zone'))
        hosts = self.client.list_hosts()['hosts']
        host = hosts[0]
        hosts = self.client.list_hosts(zone=host['zone'])['hosts']
        self.assertNotEmpty(hosts)
        self.assertIn(host, hosts)

    @decorators.idempotent_id('9af3c171-fbf4-4150-a624-22109733c2a6')
    def test_list_hosts_with_a_blank_zone(self):
        """Listing nova hosts with blank availability zone

        If send the request with a blank zone, the request will be successful
        and it will return all the hosts list
        """
        hosts = self.client.list_hosts(zone='')['hosts']
        self.assertNotEmpty(hosts)

    @decorators.idempotent_id('c6ddbadb-c94e-4500-b12f-8ffc43843ff8')
    def test_list_hosts_with_nonexistent_zone(self):
        """Listing nova hosts with not existing availability zone.

        If send the request with a nonexistent zone, the request will be
        successful and no hosts will be returned
        """
        hosts = self.client.list_hosts(zone='xxx')['hosts']
        self.assertEmpty(hosts)

    @decorators.idempotent_id('38adbb12-aee2-4498-8aec-329c72423aa4')
    def test_show_host_detail(self):
        """Showing nova host details"""
        hosts = self.client.list_hosts()['hosts']

        hosts = [host for host in hosts if host['service'] == 'compute']
        self.assertNotEmpty(hosts)

        for host in hosts:
            hostname = host['host_name']
            resources = self.client.show_host(hostname)['host']
            self.assertNotEmpty(resources)
            host_resource = resources[0]['resource']
            self.assertEqual(hostname, host_resource['host'])



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class IdentityV3ProjectsTest(base.BaseIdentityV3Test):
    """Test identity projects"""

    credentials = ['primary', 'alt']

    @decorators.idempotent_id('86128d46-e170-4644-866a-cc487f699e1d')
    def test_list_projects_returns_only_authorized_projects(self):
        """Test listing projects only returns authorized projects"""
        alt_project_name = self.os_alt.credentials.project_name
        resp = self.non_admin_users_client.list_user_projects(
            self.os_primary.credentials.user_id)

        # check that user can see only that projects that he presents in so
        # user can successfully authenticate using his credentials and
        # project name from received projects list
        for project in resp['projects']:
            # 'user_domain_id' needs to be specified otherwise tempest.lib
            # assumes it to be 'default'
            token_id, body = self.non_admin_token.get_token(
                username=self.os_primary.credentials.username,
                user_domain_id=self.os_primary.credentials.user_domain_id,
                password=self.os_primary.credentials.password,
                project_name=project['name'],
                project_domain_id=project['domain_id'],
                auth_data=True)
            self.assertNotEmpty(token_id)
            self.assertEqual(body['project']['id'], project['id'])
            self.assertEqual(body['project']['name'], project['name'])
            self.assertEqual(
                body['user']['id'], self.os_primary.credentials.user_id)

        # check that user cannot log in to alt user's project
        self.assertRaises(
            lib_exc.Unauthorized,
            self.non_admin_token.get_token,
            username=self.os_primary.credentials.username,
            user_domain_id=self.os_primary.credentials.user_domain_id,
            password=self.os_primary.credentials.password,
            project_name=alt_project_name,
            project_domain_id=project['domain_id'])



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
https://docs.openstack.org/api-ref/identity/v3/index.html#credentials
"""

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class CredentialsClient(rest_client.RestClient):
    api_version = "v3"

    def create_credential(self, **kwargs):
        """Creates a credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#create-credential
        """
        post_body = json.dumps({'credential': kwargs})
        resp, body = self.post('credentials', post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_credential(self, credential_id, **kwargs):
        """Updates a credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-credential
        """
        post_body = json.dumps({'credential': kwargs})
        resp, body = self.patch('credentials/%s' % credential_id, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_credential(self, credential_id):
        """To GET Details of a credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-credential-details
        """
        resp, body = self.get('credentials/%s' % credential_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_credentials(self, **params):
        """Lists out all the available credentials.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#list-credentials
        """
        url = 'credentials'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_credential(self, credential_id):
        """Deletes a credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/#delete-credential
        """
        resp, body = self.delete('credentials/%s' % credential_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import tenant_usages_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestTenantUsagesClient(base.BaseServiceTest):

    FAKE_SERVER_USAGES = [{
        "ended_at": None,
        "flavor": "m1.tiny",
        "hours": 1.0,
        "instance_id": "1f1deceb-17b5-4c04-84c7-e0d4499c8fe0",
        "local_gb": 1,
        "memory_mb": 512,
        "name": "new-server-test",
        "started_at": "2012-10-08T20:10:44.541277",
        "state": "active",
        "tenant_id": "openstack",
        "uptime": 3600,
        "vcpus": 1
        }]

    FAKE_TENANT_USAGES = [{
        "server_usages": FAKE_SERVER_USAGES,
        "start": "2012-10-08T21:10:44.587336",
        "stop": "2012-10-08T22:10:44.587336",
        "tenant_id": "openstack",
        "total_hours": 1,
        "total_local_gb_usage": 1,
        "total_memory_mb_usage": 512,
        "total_vcpus_usage": 1
        }]

    def setUp(self):
        super(TestTenantUsagesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = tenant_usages_client.TenantUsagesClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_tenant_usages(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_tenant_usages,
            'tempest.lib.common.rest_client.RestClient.get',
            {"tenant_usages": self.FAKE_TENANT_USAGES},
            to_utf=bytes_body)

    def test_list_tenant_usages_with_str_body(self):
        self._test_list_tenant_usages()

    def test_list_tenant_usages_with_bytes_body(self):
        self._test_list_tenant_usages(bytes_body=True)

    def _test_show_tenant_usage(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_tenant_usage,
            'tempest.lib.common.rest_client.RestClient.get',
            {"tenant_usage": self.FAKE_TENANT_USAGES[0]},
            to_utf=bytes_body,
            tenant_id='openstack')

    def test_show_tenant_usage_with_str_body(self):
        self._test_show_tenant_usage()

    def test_show_tenant_usage_with_bytes_body(self):
        self._test_show_tenant_usage(bytes_body=True)



# Copyright 2013 IBM Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class FixedIPsTestJson(base.BaseV2ComputeAdminTest):
    """Test fixed ips API"""

    @classmethod
    def skip_checks(cls):
        super(FixedIPsTestJson, cls).skip_checks()
        if CONF.service_available.neutron:
            msg = ("%s skipped as neutron is available" % cls.__name__)
            raise cls.skipException(msg)
        if not utils.get_service_list()['network']:
            raise cls.skipException("network service not enabled.")

    @classmethod
    def setup_clients(cls):
        super(FixedIPsTestJson, cls).setup_clients()
        cls.client = cls.os_admin.fixed_ips_client

    @classmethod
    def resource_setup(cls):
        super(FixedIPsTestJson, cls).resource_setup()
        server = cls.create_test_server(wait_until='ACTIVE')
        server = cls.servers_client.show_server(server['id'])['server']
        cls.ip = None
        for ip_set in server['addresses']:
            for ip in server['addresses'][ip_set]:
                if ip['OS-EXT-IPS:type'] == 'fixed':
                    cls.ip = ip['addr']
                    break
            if cls.ip:
                break
        if cls.ip is None:
            raise cls.skipException("No fixed ip found for server: %s"
                                    % server['id'])

    @decorators.idempotent_id('16b7d848-2f7c-4709-85a3-2dfb4576cc52')
    def test_list_fixed_ip_details(self):
        """Test getting fixed ip details"""
        fixed_ip = self.client.show_fixed_ip(self.ip)
        self.assertEqual(fixed_ip['fixed_ip']['address'], self.ip)

    @decorators.idempotent_id('5485077b-7e46-4cec-b402-91dc3173433b')
    def test_set_reserve(self):
        """Test reserving fixed ip"""
        self.client.reserve_fixed_ip(self.ip, reserve="None")

    @decorators.idempotent_id('7476e322-b9ff-4710-bf82-49d51bac6e2e')
    def test_set_unreserve(self):
        """Test unreserving fixed ip"""
        self.client.reserve_fixed_ip(self.ip, unreserve="None")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.identity import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class InheritsV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test keystone inherits"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def skip_checks(cls):
        super(InheritsV3TestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('OS-INHERIT', 'identity'):
            raise cls.skipException("Inherits aren't enabled")

    @classmethod
    def resource_setup(cls):
        super(InheritsV3TestJSON, cls).resource_setup()
        u_name = data_utils.rand_name('user-')
        u_desc = '%s description' % u_name
        u_email = '%s@testmail.tm' % u_name
        u_password = data_utils.rand_password()
        cls.domain = cls.create_domain()
        cls.project = cls.projects_client.create_project(
            data_utils.rand_name('project-'),
            description=data_utils.rand_name('project-desc-'),
            domain_id=cls.domain['id'])['project']
        cls.addClassResourceCleanup(cls.projects_client.delete_project,
                                    cls.project['id'])
        cls.group = cls.groups_client.create_group(
            name=data_utils.rand_name('group-'), project_id=cls.project['id'],
            domain_id=cls.domain['id'])['group']
        cls.addClassResourceCleanup(cls.groups_client.delete_group,
                                    cls.group['id'])
        if not CONF.identity_feature_enabled.immutable_user_source:
            cls.user = cls.users_client.create_user(
                name=u_name,
                description=u_desc,
                password=u_password,
                email=u_email,
                project_id=cls.project['id'],
                domain_id=cls.domain['id']
            )['user']
            cls.addClassResourceCleanup(cls.users_client.delete_user,
                                        cls.user['id'])

    def _list_assertions(self, body, fetched_role_ids, role_id):
        self.assertEqual(len(body), 1)
        self.assertIn(role_id, fetched_role_ids)

    @decorators.idempotent_id('4e6f0366-97c8-423c-b2be-41eae6ac91c8')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_inherit_assign_list_check_revoke_roles_on_domains_user(self):
        """Test assign/list/check/revoke inherited role on domain user"""
        # Create role
        src_role = self.setup_test_role()
        # Assign role on domains user
        self.inherited_roles_client.create_inherited_role_on_domains_user(
            self.domain['id'], self.user['id'], src_role['id'])
        # list role on domains user
        roles = self.inherited_roles_client.\
            list_inherited_project_role_for_user_on_domain(
                self.domain['id'], self.user['id'])['roles']

        fetched_role_ids = [i['id'] for i in roles]
        self._list_assertions(roles, fetched_role_ids,
                              src_role['id'])

        # Check role on domains user
        (self.inherited_roles_client.
         check_user_inherited_project_role_on_domain(
             self.domain['id'], self.user['id'], src_role['id']))
        # Revoke role from domains user.
        self.inherited_roles_client.delete_inherited_role_from_user_on_domain(
            self.domain['id'], self.user['id'], src_role['id'])

    @decorators.idempotent_id('c7a8dda2-be50-4fb4-9a9c-e830771078b1')
    def test_inherit_assign_list_check_revoke_roles_on_domains_group(self):
        """Test assign/list/check/revoke inherited role on domain group"""
        # Create role
        src_role = self.setup_test_role()
        # Assign role on domains group
        self.inherited_roles_client.create_inherited_role_on_domains_group(
            self.domain['id'], self.group['id'], src_role['id'])
        # List role on domains group
        roles = self.inherited_roles_client.\
            list_inherited_project_role_for_group_on_domain(
                self.domain['id'], self.group['id'])['roles']

        fetched_role_ids = [i['id'] for i in roles]
        self._list_assertions(roles, fetched_role_ids,
                              src_role['id'])

        # Check role on domains group
        (self.inherited_roles_client.
         check_group_inherited_project_role_on_domain(
             self.domain['id'], self.group['id'], src_role['id']))
        # Revoke role from domains group
        self.inherited_roles_client.delete_inherited_role_from_group_on_domain(
            self.domain['id'], self.group['id'], src_role['id'])

    @decorators.idempotent_id('18b70e45-7687-4b72-8277-b8f1a47d7591')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_inherit_assign_check_revoke_roles_on_projects_user(self):
        """Test assign/list/check/revoke inherited role on project user"""
        # Create role
        src_role = self.setup_test_role()
        # Assign role on projects user
        self.inherited_roles_client.create_inherited_role_on_projects_user(
            self.project['id'], self.user['id'], src_role['id'])
        # Check role on projects user
        (self.inherited_roles_client.
         check_user_has_flag_on_inherited_to_project(
             self.project['id'], self.user['id'], src_role['id']))
        # Revoke role from projects user
        self.inherited_roles_client.delete_inherited_role_from_user_on_project(
            self.project['id'], self.user['id'], src_role['id'])

    @decorators.idempotent_id('26021436-d5a4-4256-943c-ded01e0d4b45')
    def test_inherit_assign_check_revoke_roles_on_projects_group(self):
        """Test assign/list/check/revoke inherited role on project group"""
        # Create role
        src_role = self.setup_test_role()
        # Assign role on projects group
        self.inherited_roles_client.create_inherited_role_on_projects_group(
            self.project['id'], self.group['id'], src_role['id'])
        # Check role on projects group
        (self.inherited_roles_client.
         check_group_has_flag_on_inherited_to_project(
             self.project['id'], self.group['id'], src_role['id']))
        # Revoke role from projects group
        (self.inherited_roles_client.
         delete_inherited_role_from_group_on_project(
             self.project['id'], self.group['id'], src_role['id']))

    @decorators.idempotent_id('3acf666e-5354-42ac-8e17-8b68893bcd36')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_inherit_assign_list_revoke_user_roles_on_domain(self):
        """Test assign/list/check/revoke inherited role on domain"""
        # Create role
        src_role = self.setup_test_role()

        # Create a project hierarchy
        leaf_project = self.setup_test_project(domain_id=self.domain['id'],
                                               parent_id=self.project['id'])

        # Assign role on domain
        self.inherited_roles_client.create_inherited_role_on_domains_user(
            self.domain['id'], self.user['id'], src_role['id'])

        # List "effective" role assignments from user on the parent project
        params = {'scope.project.id': self.project['id'],
                  'user.id': self.user['id']}
        assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertNotEmpty(assignments)

        # List "effective" role assignments from user on the leaf project
        params['scope.project.id'] = leaf_project['id']
        assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertNotEmpty(assignments)

        # Revoke role from domain
        self.inherited_roles_client.delete_inherited_role_from_user_on_domain(
            self.domain['id'], self.user['id'], src_role['id'])

        # List "effective" role assignments from user on the parent project
        # should return an empty list
        params['scope.project.id'] = self.project['id']
        assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertEmpty(assignments)

        # List "effective" role assignments from user on the leaf project
        # should return an empty list
        params['scope.project.id'] = leaf_project['id']
        assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertEmpty(assignments)

    @decorators.idempotent_id('9f02ccd9-9b57-46b4-8f77-dd5a736f3a06')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    def test_inherit_assign_list_revoke_user_roles_on_project_tree(self):
        """Test assign/list/check/revoke inherited role on project tree"""
        # Create role
        src_role = self.setup_test_role()

        # Create a project hierarchy
        leaf_project = self.setup_test_project(domain_id=self.domain['id'],
                                               parent_id=self.project['id'])

        # Assign role on parent project
        self.inherited_roles_client.create_inherited_role_on_projects_user(
            self.project['id'], self.user['id'], src_role['id'])

        # List "effective" role assignments from user on the leaf project
        params = {'scope.project.id': leaf_project['id'],
                  'user.id': self.user['id']}
        assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertNotEmpty(assignments)

        # Revoke role from parent project
        self.inherited_roles_client.delete_inherited_role_from_user_on_project(
            self.project['id'], self.user['id'], src_role['id'])

        # List "effective" role assignments from user on the leaf project
        # should return an empty list
        assignments = self.role_assignments.list_role_assignments(
            effective=True, **params)['role_assignments']
        self.assertEmpty(assignments)



# Copyright 2014 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from io import StringIO
import socket
from unittest import mock

import six
import testtools

from tempest.lib.common import ssh
from tempest.lib import exceptions
from tempest.tests import base
import tempest.tests.utils as utils


class TestSshClient(base.TestCase):

    SELECT_POLLIN = 1

    @mock.patch('paramiko.RSAKey.from_private_key')
    @mock.patch('io.StringIO')
    def test_pkey_calls_paramiko_RSAKey(self, cs_mock, rsa_mock):
        cs_mock.return_value = mock.sentinel.csio
        pkey = 'mykey'
        ssh.Client('localhost', 'root', pkey=pkey)
        rsa_mock.assert_called_once_with(mock.sentinel.csio)
        cs_mock.assert_called_once_with('mykey')
        rsa_mock.reset_mock()
        cs_mock.reset_mock()
        pkey = mock.sentinel.pkey
        # Shouldn't call out to load a file from RSAKey, since
        # a sentinel isn't a basestring...
        ssh.Client('localhost', 'root', pkey=pkey)
        self.assertEqual(0, rsa_mock.call_count)
        self.assertEqual(0, cs_mock.call_count)

    def _set_ssh_connection_mocks(self):
        client_mock = mock.MagicMock()
        client_mock.connect.return_value = True
        return (self.patch('paramiko.SSHClient'),
                self.patch('paramiko.AutoAddPolicy'),
                client_mock)

    def test_get_ssh_connection(self):
        c_mock, aa_mock, client_mock = self._set_ssh_connection_mocks()
        s_mock = self.patch('time.sleep')

        c_mock.return_value = client_mock
        aa_mock.return_value = mock.sentinel.aa

        # Test normal case for successful connection on first try
        client = ssh.Client('localhost', 'root', timeout=2)
        client._get_ssh_connection(sleep=1)

        aa_mock.assert_called_once_with()
        client_mock.set_missing_host_key_policy.assert_called_once_with(
            mock.sentinel.aa)
        expected_connect = [mock.call(
            'localhost',
            port=22,
            username='root',
            pkey=None,
            key_filename=None,
            look_for_keys=False,
            timeout=10.0,
            password=None,
            sock=None
        )]
        self.assertEqual(expected_connect, client_mock.connect.mock_calls)
        self.assertEqual(0, s_mock.call_count)

    def test_get_ssh_connection_over_ssh(self):
        c_mock, aa_mock, client_mock = self._set_ssh_connection_mocks()
        proxy_client_mock = mock.MagicMock()
        proxy_client_mock.connect.return_value = True
        s_mock = self.patch('time.sleep')

        c_mock.side_effect = [client_mock, proxy_client_mock]
        aa_mock.return_value = mock.sentinel.aa

        proxy_client = ssh.Client('proxy-host', 'proxy-user', timeout=2)
        client = ssh.Client('localhost', 'root', timeout=2,
                            proxy_client=proxy_client)
        client._get_ssh_connection(sleep=1)

        aa_mock.assert_has_calls([mock.call(), mock.call()])
        proxy_client_mock.set_missing_host_key_policy.assert_called_once_with(
            mock.sentinel.aa)
        proxy_expected_connect = [mock.call(
            'proxy-host',
            port=22,
            username='proxy-user',
            pkey=None,
            key_filename=None,
            look_for_keys=False,
            timeout=10.0,
            password=None,
            sock=None
        )]
        self.assertEqual(proxy_expected_connect,
                         proxy_client_mock.connect.mock_calls)
        client_mock.set_missing_host_key_policy.assert_called_once_with(
            mock.sentinel.aa)
        expected_connect = [mock.call(
            'localhost',
            port=22,
            username='root',
            pkey=None,
            key_filename=None,
            look_for_keys=False,
            timeout=10.0,
            password=None,
            sock=proxy_client_mock.get_transport().open_session()
        )]
        self.assertEqual(expected_connect, client_mock.connect.mock_calls)
        self.assertEqual(0, s_mock.call_count)

    @mock.patch('time.sleep')
    def test_get_ssh_connection_two_attemps(self, sleep_mock):
        c_mock, aa_mock, client_mock = self._set_ssh_connection_mocks()

        c_mock.return_value = client_mock
        client_mock.connect.side_effect = [
            socket.error,
            mock.MagicMock()
        ]

        client = ssh.Client('localhost', 'root', timeout=1)
        client._get_ssh_connection(sleep=1)
        # We slept 2 seconds: because sleep is "1" and backoff is "1" too
        sleep_mock.assert_called_once_with(2)
        self.assertEqual(2, client_mock.connect.call_count)

    def test_get_ssh_connection_timeout(self):
        c_mock, aa_mock, client_mock = self._set_ssh_connection_mocks()

        timeout = 2
        time_mock = self.patch('time.time')
        time_mock.side_effect = utils.generate_timeout_series(timeout + 1)

        c_mock.return_value = client_mock
        client_mock.connect.side_effect = [
            socket.error,
            socket.error,
            socket.error,
        ]

        client = ssh.Client('localhost', 'root', timeout=timeout)
        # We need to mock LOG here because LOG.info() calls time.time()
        # in order to preprend a timestamp.
        with mock.patch.object(ssh, 'LOG'):
            self.assertRaises(exceptions.SSHTimeout,
                              client._get_ssh_connection)

        # time.time() should be called twice, first to start the timer
        # and then to compute the timedelta
        self.assertEqual(2, time_mock.call_count)

    @mock.patch('select.POLLIN', SELECT_POLLIN, create=True)
    def test_timeout_in_exec_command(self):
        chan_mock, poll_mock, _, _ = (
            self._set_mocks_for_select([0, 0, 0], True))

        # Test for a timeout condition immediately raised
        client = ssh.Client('localhost', 'root', timeout=2)
        with testtools.ExpectedException(exceptions.TimeoutException):
            client.exec_command("test")

        chan_mock.fileno.assert_called_once_with()
        chan_mock.exec_command.assert_called_once_with("test")
        chan_mock.shutdown_write.assert_called_once_with()

        poll_mock.register.assert_called_once_with(
            chan_mock, self.SELECT_POLLIN)
        poll_mock.poll.assert_called_once_with(10)

    @mock.patch('select.POLLIN', SELECT_POLLIN, create=True)
    def test_exec_command(self):
        chan_mock, poll_mock, select_mock, client_mock = (
            self._set_mocks_for_select([[1, 0, 0]], True))

        chan_mock.recv_exit_status.return_value = 0
        chan_mock.recv.return_value = b''
        chan_mock.recv_stderr.return_value = b''

        client = ssh.Client('localhost', 'root', timeout=2)
        client.exec_command("test")

        chan_mock.fileno.assert_called_once_with()
        chan_mock.exec_command.assert_called_once_with("test")
        chan_mock.shutdown_write.assert_called_once_with()

        select_mock.assert_called_once_with()
        poll_mock.register.assert_called_once_with(
            chan_mock, self.SELECT_POLLIN)
        poll_mock.poll.assert_called_once_with(10)
        chan_mock.recv_ready.assert_called_once_with()
        chan_mock.recv.assert_called_once_with(1024)
        chan_mock.recv_stderr_ready.assert_called_once_with()
        chan_mock.recv_stderr.assert_called_once_with(1024)
        chan_mock.recv_exit_status.assert_called_once_with()

        client_mock.close.assert_called_once_with()

    def _set_mocks_for_select(self, poll_data, ito_value=False):
        gsc_mock = self.patch('tempest.lib.common.ssh.Client.'
                              '_get_ssh_connection')
        ito_mock = self.patch('tempest.lib.common.ssh.Client._is_timed_out')
        csp_mock = self.patch(
            'tempest.lib.common.ssh.Client._can_system_poll')
        csp_mock.return_value = True

        select_mock = self.patch('select.poll', create=True)
        client_mock = mock.MagicMock()
        tran_mock = mock.MagicMock()
        chan_mock = mock.MagicMock()
        poll_mock = mock.MagicMock()

        select_mock.return_value = poll_mock
        gsc_mock.return_value = client_mock
        ito_mock.return_value = ito_value
        client_mock.get_transport.return_value = tran_mock
        tran_mock.open_session().__enter__.return_value = chan_mock
        if isinstance(poll_data[0], list):
            poll_mock.poll.side_effect = poll_data
        else:
            poll_mock.poll.return_value = poll_data

        return chan_mock, poll_mock, select_mock, client_mock

    _utf8_string = six.unichr(1071)
    _utf8_bytes = _utf8_string.encode("utf-8")

    @mock.patch('select.POLLIN', SELECT_POLLIN, create=True)
    def test_exec_good_command_output(self):
        chan_mock, poll_mock, _, _ = (
            self._set_mocks_for_select([1, 0, 0]))
        closed_prop = mock.PropertyMock(return_value=True)
        type(chan_mock).closed = closed_prop

        chan_mock.recv_exit_status.return_value = 0
        chan_mock.recv.side_effect = [self._utf8_bytes[0:1],
                                      self._utf8_bytes[1:], b'R', b'']
        chan_mock.recv_stderr.return_value = b''

        client = ssh.Client('localhost', 'root', timeout=2)
        out_data = client.exec_command("test")
        self.assertEqual(self._utf8_string + 'R', out_data)

    @mock.patch('select.POLLIN', SELECT_POLLIN, create=True)
    def test_exec_bad_command_output(self):
        chan_mock, poll_mock, _, _ = (
            self._set_mocks_for_select([1, 0, 0]))
        closed_prop = mock.PropertyMock(return_value=True)
        type(chan_mock).closed = closed_prop

        chan_mock.recv_exit_status.return_value = 1
        chan_mock.recv.return_value = b''
        chan_mock.recv_stderr.side_effect = [b'R', self._utf8_bytes[0:1],
                                             self._utf8_bytes[1:], b'']

        client = ssh.Client('localhost', 'root', timeout=2)
        exc = self.assertRaises(exceptions.SSHExecCommandFailed,
                                client.exec_command, "test")
        self.assertIn('R' + self._utf8_string, str(exc))

    def test_exec_command_no_select(self):
        gsc_mock = self.patch('tempest.lib.common.ssh.Client.'
                              '_get_ssh_connection')
        csp_mock = self.patch(
            'tempest.lib.common.ssh.Client._can_system_poll')
        csp_mock.return_value = False

        select_mock = self.patch('select.poll', create=True)
        client_mock = mock.MagicMock()
        tran_mock = mock.MagicMock()
        chan_mock = mock.MagicMock()

        # Test for proper reading of STDOUT and STDERROR

        gsc_mock.return_value = client_mock
        client_mock.get_transport.return_value = tran_mock
        tran_mock.open_session().__enter__.return_value = chan_mock
        chan_mock.recv_exit_status.return_value = 0

        std_out_mock = mock.MagicMock(StringIO)
        std_err_mock = mock.MagicMock(StringIO)
        chan_mock.makefile.return_value = std_out_mock
        chan_mock.makefile_stderr.return_value = std_err_mock

        client = ssh.Client('localhost', 'root', timeout=2)
        client.exec_command("test")

        chan_mock.makefile.assert_called_once_with('rb', 1024)
        chan_mock.makefile_stderr.assert_called_once_with('rb', 1024)
        std_out_mock.read.assert_called_once_with()
        std_err_mock.read.assert_called_once_with()
        self.assertFalse(select_mock.called)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators
from tempest.scenario import manager


class TestObjectStorageBasicOps(manager.ObjectStorageScenarioTest):
    @decorators.idempotent_id('b920faf1-7b8a-4657-b9fe-9c4512bfb381')
    @utils.services('object_storage')
    def test_swift_basic_ops(self):
        """Test swift basic ops.

        * get swift stat.
        * create container.
        * upload a file to the created container.
        * list container's objects and assure that the uploaded file is
          present.
        * download the object and check the content
        * delete object from container.
        * list container's objects and assure that the deleted file is gone.
        * delete a container.
        """
        self.get_swift_stat()
        container_name = self.create_container()
        obj_name, obj_data = self.upload_object_to_container(container_name)
        self.list_and_check_container_objects(container_name,
                                              present_obj=[obj_name])
        self.download_and_verify(container_name, obj_name, obj_data)
        self.delete_object(container_name, obj_name)
        self.list_and_check_container_objects(container_name,
                                              not_present_obj=[obj_name])
        self.delete_container(container_name)

    @decorators.idempotent_id('916c7111-cb1f-44b2-816d-8f760e4ea910')
    @decorators.attr(type='slow')
    @utils.services('object_storage')
    def test_swift_acl_anonymous_download(self):
        """This test will cover below steps:

        1. Create container
        2. Upload object to the new container
        3. Change the ACL of the container
        4. Check if the object can be download by anonymous user
        5. Delete the object and container
        """
        container_name = self.create_container()
        obj_name, obj_data = self.upload_object_to_container(container_name)
        obj_url = '%s/%s/%s' % (self.object_client.base_url,
                                container_name, obj_name)
        resp, _ = self.object_client.raw_request(obj_url, 'GET')
        self.assertEqual(resp.status, 401)
        metadata_param = {'X-Container-Read': '.r:*'}
        self.container_client.create_update_or_delete_container_metadata(
            container_name, create_update_metadata=metadata_param,
            create_update_metadata_prefix='')
        resp, _ = self.container_client.list_container_metadata(container_name)
        self.assertEqual(metadata_param['X-Container-Read'],
                         resp['x-container-read'])
        resp, data = self.object_client.raw_request(obj_url, 'GET')
        self.assertEqual(resp.status, 200)
        self.assertEqual(obj_data, data)



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import hashlib
import hmac
import time
from urllib import parse as urlparse

from tempest.api.object_storage import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ObjectFormPostNegativeTest(base.BaseObjectTest):
    """Negative tests of object post with form"""

    metadata = {}
    containers = []

    @classmethod
    def resource_setup(cls):
        super(ObjectFormPostNegativeTest, cls).resource_setup()
        cls.container_name = cls.create_container()
        cls.object_name = data_utils.rand_name(name='ObjectTemp')

        cls.key = 'Meta'
        cls.metadata = {'Temp-URL-Key': cls.key}
        cls.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=cls.metadata)

    def setUp(self):
        super(ObjectFormPostNegativeTest, self).setUp()

        # make sure the metadata has been set
        account_client_metadata, _ = \
            self.account_client.list_account_metadata()
        self.assertIn('x-account-meta-temp-url-key',
                      account_client_metadata)
        self.assertEqual(
            account_client_metadata['x-account-meta-temp-url-key'],
            self.key)

    @classmethod
    def resource_cleanup(cls):
        cls.account_client.create_update_or_delete_account_metadata(
            delete_metadata=cls.metadata)
        cls.delete_containers()
        super(ObjectFormPostNegativeTest, cls).resource_cleanup()

    def get_multipart_form(self, expires=600):
        path = "%s/%s/%s" % (
            urlparse.urlparse(self.container_client.base_url).path,
            self.container_name,
            self.object_name)

        redirect = ''
        max_file_size = 104857600
        max_file_count = 10
        expires += int(time.time())
        hmac_body = '%s\n%s\n%s\n%s\n%s' % (path,
                                            redirect,
                                            max_file_size,
                                            max_file_count,
                                            expires)

        signature = hmac.new(
            self.key.encode(), hmac_body.encode(), hashlib.sha1
        ).hexdigest()

        fields = {'redirect': redirect,
                  'max_file_size': str(max_file_size),
                  'max_file_count': str(max_file_count),
                  'expires': str(expires),
                  'signature': signature}

        boundary = '--boundary--'
        data = []
        for (key, value) in fields.items():
            data.append('--' + boundary)
            data.append('Content-Disposition: form-data; name="%s"' % key)
            data.append('')
            data.append(value)

        data.append('--' + boundary)
        data.append('Content-Disposition: form-data; '
                    'name="file1"; filename="testfile"')
        data.append('Content-Type: application/octet-stream')
        data.append('')
        data.append('hello world')

        data.append('--' + boundary + '--')
        data.append('')

        body = '\r\n'.join(data)
        content_type = 'multipart/form-data; boundary=%s' % boundary
        return body, content_type

    @decorators.idempotent_id('d3fb3c4d-e627-48ce-9379-a1631f21336d')
    @utils.requires_ext(extension='formpost', service='object')
    @decorators.attr(type=['negative'])
    def test_post_object_using_form_expired(self):
        """Test posting object using expired form"""
        body, content_type = self.get_multipart_form(expires=1)
        time.sleep(2)

        headers = {'Content-Type': content_type,
                   'Content-Length': str(len(body))}

        url = "%s/%s" % (self.container_name, self.object_name)
        exc = self.assertRaises(
            lib_exc.Unauthorized,
            self.object_client.post,
            url, body, headers=headers)
        self.assertIn('FormPost: Form Expired', str(exc))

    @decorators.idempotent_id('b277257f-113c-4499-b8d1-5fead79f7360')
    @utils.requires_ext(extension='formpost', service='object')
    @decorators.attr(type=['negative'])
    def test_post_object_using_form_invalid_signature(self):
        """Test posting object using form with invalid signature"""
        self.key = "Wrong"
        body, content_type = self.get_multipart_form()

        headers = {'Content-Type': content_type,
                   'Content-Length': str(len(body))}

        url = "%s/%s" % (self.container_name, self.object_name)
        exc = self.assertRaises(
            lib_exc.Unauthorized,
            self.object_client.post,
            url, body, headers=headers)
        self.assertIn('FormPost: Invalid Signature', str(exc))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urlparse

from oslo_log import log as logging
import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest.common.utils.linux import remote_client
from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF

LOG = logging.getLogger(__name__)


class ServerActionsTestJSON(base.BaseV2ComputeTest):
    """Test server actions"""

    def setUp(self):
        # NOTE(afazekas): Normally we use the same server with all test cases,
        # but if it has an issue, we build a new one
        super(ServerActionsTestJSON, self).setUp()
        # Check if the server is in a clean state after test
        try:
            waiters.wait_for_server_status(self.client,
                                           self.server_id, 'ACTIVE')
        except lib_exc.NotFound:
            # The server was deleted by previous test, create a new one
            # Use class level validation resources to avoid them being
            # deleted once a test is over
            validation_resources = self.get_class_validation_resources(
                self.os_primary)
            server = self.create_test_server(
                validatable=True,
                validation_resources=validation_resources,
                wait_until='ACTIVE')
            self.__class__.server_id = server['id']
        except Exception:
            # Rebuild server if something happened to it during a test
            self.__class__.server_id = self.recreate_server(
                self.server_id, validatable=True)

    def tearDown(self):
        super(ServerActionsTestJSON, self).tearDown()
        # NOTE(zhufl): Because server_check_teardown will raise Exception
        # which will prevent other cleanup steps from being executed, so
        # server_check_teardown should be called after super's tearDown.
        self.server_check_teardown()

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(ServerActionsTestJSON, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServerActionsTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServerActionsTestJSON, cls).resource_setup()
        cls.server_id = cls.recreate_server(None, validatable=True)

    @decorators.idempotent_id('6158df09-4b82-4ab3-af6d-29cf36af858d')
    @testtools.skipUnless(CONF.compute_feature_enabled.change_password,
                          'Change password not available.')
    def test_change_server_password(self):
        """Test changing server's password

        The server's password should be set to the provided password and
        the user can authenticate with the new password.
        """
        # Since this test messes with the password and makes the
        # server unreachable, it should create its own server
        validation_resources = self.get_test_validation_resources(
            self.os_primary)
        newserver = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='ACTIVE')
        self.addCleanup(self.delete_server, newserver['id'])
        # The server's password should be set to the provided password
        new_password = 'Newpass1234'
        self.client.change_password(newserver['id'], adminPass=new_password)
        waiters.wait_for_server_status(self.client, newserver['id'], 'ACTIVE')

        if CONF.validation.run_validation:
            # Verify that the user can authenticate with the new password
            server = self.client.show_server(newserver['id'])['server']
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.ssh_user,
                new_password,
                server=server,
                servers_client=self.client)
            linux_client.validate_authentication()

    def _test_reboot_server(self, reboot_type):
        if CONF.validation.run_validation:
            validation_resources = self.get_class_validation_resources(
                self.os_primary)
            # Get the time the server was last rebooted,
            server = self.client.show_server(self.server_id)['server']
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.ssh_user,
                self.password,
                validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.client)
            boot_time = linux_client.get_boot_time()

            # NOTE: This sync is for avoiding the loss of pub key data
            # in a server
            linux_client.exec_command("sync")

        self.client.reboot_server(self.server_id, type=reboot_type)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')

        if CONF.validation.run_validation:
            # Log in and verify the boot time has changed
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.ssh_user,
                self.password,
                validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.client)
            new_boot_time = linux_client.get_boot_time()
            self.assertGreater(new_boot_time, boot_time,
                               '%s > %s' % (new_boot_time, boot_time))

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2cb1baf6-ac8d-4429-bf0d-ba8a0ba53e32')
    def test_reboot_server_hard(self):
        """Test hard rebooting server

        The server should be power cycled.
        """
        self._test_reboot_server('HARD')

    @decorators.idempotent_id('1d1c9104-1b0a-11e7-a3d4-fa163e65f5ce')
    def test_remove_server_all_security_groups(self):
        """Test removing all security groups from server"""
        server = self.create_test_server(wait_until='ACTIVE')

        # Remove all Security group
        self.client.remove_security_group(
            server['id'], name=server['security_groups'][0]['name'])

        # Verify all Security group
        server = self.client.show_server(server['id'])['server']
        self.assertNotIn('security_groups', server)

    def _rebuild_server_and_check(self, image_ref):
        rebuilt_server = (self.client.rebuild_server(self.server_id, image_ref)
                          ['server'])
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        msg = ('Server was not rebuilt to the original image. '
               'The original image: {0}. The current image: {1}'
               .format(image_ref, rebuilt_server['image']['id']))
        self.assertEqual(image_ref, rebuilt_server['image']['id'], msg)

    def _test_rebuild_server(self):
        # Get the IPs the server has before rebuilding it
        original_addresses = (self.client.show_server(self.server_id)['server']
                              ['addresses'])
        # The server should be rebuilt using the provided image and data
        meta = {'rebuild': 'server'}
        new_name = data_utils.rand_name(self.__class__.__name__ + '-server')
        password = 'rebuildPassw0rd'
        rebuilt_server = self.client.rebuild_server(
            self.server_id,
            self.image_ref_alt,
            name=new_name,
            metadata=meta,
            adminPass=password)['server']

        # If the server was rebuilt on a different image, restore it to the
        # original image once the test ends
        if self.image_ref_alt != self.image_ref:
            self.addCleanup(self._rebuild_server_and_check, self.image_ref)

        # Verify the properties in the initial response are correct
        self.assertEqual(self.server_id, rebuilt_server['id'])
        rebuilt_image_id = rebuilt_server['image']['id']
        self.assertTrue(self.image_ref_alt.endswith(rebuilt_image_id))
        self.assert_flavor_equal(self.flavor_ref, rebuilt_server['flavor'])

        # Verify the server properties after the rebuild completes
        waiters.wait_for_server_status(self.client,
                                       rebuilt_server['id'], 'ACTIVE')
        server = self.client.show_server(rebuilt_server['id'])['server']
        rebuilt_image_id = server['image']['id']
        self.assertTrue(self.image_ref_alt.endswith(rebuilt_image_id))
        self.assertEqual(new_name, server['name'])
        self.assertEqual(original_addresses, server['addresses'])

        if CONF.validation.run_validation:
            validation_resources = self.get_class_validation_resources(
                self.os_primary)
            # Authentication is attempted in the following order of priority:
            # 1.The key passed in, if one was passed in.
            # 2.Any key we can find through an SSH agent (if allowed).
            # 3.Any "id_rsa", "id_dsa" or "id_ecdsa" key discoverable in
            #   ~/.ssh/ (if allowed).
            # 4.Plain username/password auth, if a password was given.
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(rebuilt_server, validation_resources),
                self.ssh_alt_user,
                password,
                validation_resources['keypair']['private_key'],
                server=rebuilt_server,
                servers_client=self.client)
            linux_client.validate_authentication()

    @decorators.idempotent_id('aaa6cdf3-55a7-461a-add9-1c8596b9a07c')
    def test_rebuild_server(self):
        """Test rebuilding server

        The server should be rebuilt using the provided image and data.
        """
        self._test_rebuild_server()

    @decorators.idempotent_id('30449a88-5aff-4f9b-9866-6ee9b17f906d')
    def test_rebuild_server_in_stop_state(self):
        """Test rebuilding server in stop state

        The server in stop state should be rebuilt using the provided
        image and remain in SHUTOFF state.
        """
        server = self.client.show_server(self.server_id)['server']
        old_image = server['image']['id']
        new_image = (self.image_ref_alt
                     if old_image == self.image_ref else self.image_ref)
        self.client.stop_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'SHUTOFF')
        rebuilt_server = (self.client.rebuild_server(self.server_id, new_image)
                          ['server'])
        # If the server was rebuilt on a different image, restore it to the
        # original image once the test ends
        if self.image_ref_alt != self.image_ref:
            self.addCleanup(self._rebuild_server_and_check, old_image)

        # Verify the properties in the initial response are correct
        self.assertEqual(self.server_id, rebuilt_server['id'])
        rebuilt_image_id = rebuilt_server['image']['id']
        self.assertEqual(new_image, rebuilt_image_id)
        self.assert_flavor_equal(self.flavor_ref, rebuilt_server['flavor'])

        # Verify the server properties after the rebuild completes
        waiters.wait_for_server_status(self.client,
                                       rebuilt_server['id'], 'SHUTOFF')
        server = self.client.show_server(rebuilt_server['id'])['server']
        rebuilt_image_id = server['image']['id']
        self.assertEqual(new_image, rebuilt_image_id)

        self.client.start_server(self.server_id)

    # NOTE(mriedem): Marked as slow because while rebuild and volume-backed is
    # common, we don't actually change the image (you can't with volume-backed
    # rebuild) so this isn't testing much outside normal rebuild
    # (and it's slow).
    @decorators.attr(type='slow')
    @decorators.idempotent_id('b68bd8d6-855d-4212-b59b-2e704044dace')
    @utils.services('volume')
    def test_rebuild_server_with_volume_attached(self):
        """Test rebuilding server with volume attached

        The volume should be attached to the instance after rebuild.
        """
        # create a new volume and attach it to the server
        volume = self.create_volume()

        server = self.client.show_server(self.server_id)['server']
        self.attach_volume(server, volume)

        # run general rebuild test
        self._test_rebuild_server()

        # make sure the volume is attached to the instance after rebuild
        vol_after_rebuild = self.volumes_client.show_volume(volume['id'])
        vol_after_rebuild = vol_after_rebuild['volume']
        self.assertEqual('in-use', vol_after_rebuild['status'])
        self.assertEqual(self.server_id,
                         vol_after_rebuild['attachments'][0]['server_id'])
        if CONF.validation.run_validation:
            validation_resources = self.get_class_validation_resources(
                self.os_primary)
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.ssh_alt_user,
                password=None,
                pkey=validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.client)
            linux_client.validate_authentication()

    def _test_resize_server_confirm(self, server_id, stop=False):
        # The server's RAM and disk space should be modified to that of
        # the provided flavor

        if stop:
            self.client.stop_server(server_id)
            waiters.wait_for_server_status(self.client, server_id,
                                           'SHUTOFF')

        self.client.resize_server(server_id, self.flavor_ref_alt)
        # NOTE(jlk): Explicitly delete the server to get a new one for later
        # tests. Avoids resize down race issues.
        self.addCleanup(self.delete_server, server_id)
        waiters.wait_for_server_status(self.client, server_id,
                                       'VERIFY_RESIZE')

        self.client.confirm_resize_server(server_id)
        expected_status = 'SHUTOFF' if stop else 'ACTIVE'
        waiters.wait_for_server_status(self.client, server_id,
                                       expected_status)

        server = self.client.show_server(server_id)['server']
        self.assert_flavor_equal(self.flavor_ref_alt, server['flavor'])

        if stop:
            # NOTE(mriedem): tearDown requires the server to be started.
            self.client.start_server(server_id)

    @decorators.idempotent_id('1499262a-9328-4eda-9068-db1ac57498d2')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_confirm(self):
        """Test resizing server and then confirming"""
        self._test_resize_server_confirm(self.server_id, stop=False)

    @decorators.idempotent_id('e6c28180-7454-4b59-b188-0257af08a63b')
    @decorators.related_bug('1728603')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @utils.services('volume')
    def test_resize_volume_backed_server_confirm(self):
        """Test resizing a volume backed server and then confirming"""
        # We have to create a new server that is volume-backed since the one
        # from setUp is not volume-backed.
        kwargs = {'volume_backed': True,
                  'wait_until': 'ACTIVE'}
        if CONF.validation.run_validation:
            validation_resources = self.get_test_validation_resources(
                self.os_primary)
            kwargs.update({'validatable': True,
                           'validation_resources': validation_resources})
        server = self.create_test_server(**kwargs)

        # NOTE(mgoddard): Get detailed server to ensure addresses are present
        # in fixed IP case.
        server = self.servers_client.show_server(server['id'])['server']

        self._test_resize_server_confirm(server['id'])

        if CONF.compute_feature_enabled.console_output:
            # Now do something interactive with the guest like get its console
            # output; we don't actually care about the output,
            # just that it doesn't raise an error.
            self.client.get_console_output(server['id'])
        if CONF.validation.run_validation:
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.ssh_user,
                password=None,
                pkey=validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.client)
            linux_client.validate_authentication()

    @decorators.idempotent_id('138b131d-66df-48c9-a171-64f45eb92962')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_confirm_from_stopped(self):
        """Test resizing a stopped server and then confirming"""
        self._test_resize_server_confirm(self.server_id, stop=True)

    @decorators.idempotent_id('c03aab19-adb1-44f5-917d-c419577e9e68')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_revert(self):
        """Test resizing server and then reverting

        The server's RAM and disk space should return to its original
        values after a resize is reverted.
        """

        self.client.resize_server(self.server_id, self.flavor_ref_alt)
        # NOTE(zhufl): Explicitly delete the server to get a new one for later
        # tests. Avoids resize down race issues.
        self.addCleanup(self.delete_server, self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id,
                                       'VERIFY_RESIZE')

        self.client.revert_resize_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')

        server = self.client.show_server(self.server_id)['server']
        self.assert_flavor_equal(self.flavor_ref, server['flavor'])

    @decorators.idempotent_id('fbbf075f-a812-4022-bc5c-ccb8047eef12')
    @decorators.related_bug('1737599')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @utils.services('volume')
    def test_resize_server_revert_with_volume_attached(self):
        """Test resizing a volume attached server and then reverting

        Tests attaching a volume to a server instance and then resizing
        the instance. Once the instance is resized, revert the resize which
        should move the instance and volume attachment back to the original
        compute host.
        """

        # Create a blank volume and attach it to the server created in setUp.
        volume = self.create_volume()
        server = self.client.show_server(self.server_id)['server']
        self.attach_volume(server, volume)
        # Now resize the server with the blank volume attached.
        self.client.resize_server(self.server_id, self.flavor_ref_alt)
        # Explicitly delete the server to get a new one for later
        # tests. Avoids resize down race issues.
        self.addCleanup(self.delete_server, self.server_id)
        waiters.wait_for_server_status(
            self.client, self.server_id, 'VERIFY_RESIZE')
        # Now revert the resize which should move the instance and it's volume
        # attachment back to the original source compute host.
        self.client.revert_resize_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        # Make sure everything still looks OK.
        server = self.client.show_server(self.server_id)['server']
        self.assert_flavor_equal(self.flavor_ref, server['flavor'])
        attached_volumes = server['os-extended-volumes:volumes_attached']
        self.assertEqual(1, len(attached_volumes))
        self.assertEqual(volume['id'], attached_volumes[0]['id'])

    @decorators.idempotent_id('b963d4f1-94b3-4c40-9e97-7b583f46e470')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting not available, backup not possible.')
    @utils.services('image')
    def test_create_backup(self):
        """Test creating server backup

        1. create server backup1 with rotation=2, there are 1 backup.
        2. create server backup2 with rotation=2, there are 2 backups.
        3. create server backup3, due to the rotation is 2, the first one
           (backup1) will be deleted, so now there are still 2 backups.
        """

        # create the first and the second backup

        # Check if glance v1 is available to determine which client to use. We
        # prefer glance v1 for the compute API tests since the compute image
        # API proxy was written for glance v1.
        if CONF.image_feature_enabled.api_v1:
            glance_client = self.os_primary.image_client
        elif CONF.image_feature_enabled.api_v2:
            glance_client = self.os_primary.image_client_v2
        else:
            raise lib_exc.InvalidConfiguration(
                'Either api_v1 or api_v2 must be True in '
                '[image-feature-enabled].')

        backup1 = data_utils.rand_name('backup-1')
        resp = self.client.create_backup(self.server_id,
                                         backup_type='daily',
                                         rotation=2,
                                         name=backup1)
        oldest_backup_exist = True

        # the oldest one should be deleted automatically in this test
        def _clean_oldest_backup(oldest_backup):
            if oldest_backup_exist:
                try:
                    glance_client.delete_image(oldest_backup)
                except lib_exc.NotFound:
                    pass
                else:
                    LOG.warning("Deletion of oldest backup %s should not have "
                                "been successful as it should have been "
                                "deleted during rotation.", oldest_backup)

        if api_version_utils.compare_version_header_to_response(
                "OpenStack-API-Version", "compute 2.45", resp.response, "lt"):
            image1_id = resp['image_id']
        else:
            image1_id = data_utils.parse_image_id(resp.response['location'])
        self.addCleanup(_clean_oldest_backup, image1_id)
        waiters.wait_for_image_status(glance_client,
                                      image1_id, 'active')

        backup2 = data_utils.rand_name('backup-2')
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        resp = self.client.create_backup(self.server_id,
                                         backup_type='daily',
                                         rotation=2,
                                         name=backup2)
        if api_version_utils.compare_version_header_to_response(
                "OpenStack-API-Version", "compute 2.45", resp.response, "lt"):
            image2_id = resp['image_id']
        else:
            image2_id = data_utils.parse_image_id(resp.response['location'])
        self.addCleanup(glance_client.delete_image, image2_id)
        waiters.wait_for_image_status(glance_client,
                                      image2_id, 'active')

        # verify they have been created
        properties = {
            'image_type': 'backup',
            'backup_type': "daily",
            'instance_uuid': self.server_id,
        }
        params = {
            'status': 'active',
            'sort_key': 'created_at',
            'sort_dir': 'asc'
        }
        if CONF.image_feature_enabled.api_v1:
            for key, value in properties.items():
                params['property-%s' % key] = value
            image_list = glance_client.list_images(
                detail=True,
                **params)['images']
        else:
            # Additional properties are flattened in glance v2.
            params.update(properties)
            image_list = glance_client.list_images(params)['images']

        self.assertEqual(2, len(image_list))
        self.assertEqual((backup1, backup2),
                         (image_list[0]['name'], image_list[1]['name']))

        # create the third one, due to the rotation is 2,
        # the first one will be deleted
        backup3 = data_utils.rand_name('backup-3')
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        resp = self.client.create_backup(self.server_id,
                                         backup_type='daily',
                                         rotation=2,
                                         name=backup3)
        if api_version_utils.compare_version_header_to_response(
                "OpenStack-API-Version", "compute 2.45", resp.response, "lt"):
            image3_id = resp['image_id']
        else:
            image3_id = data_utils.parse_image_id(resp.response['location'])
        self.addCleanup(glance_client.delete_image, image3_id)
        # the first back up should be deleted
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        glance_client.wait_for_resource_deletion(image1_id)
        oldest_backup_exist = False
        if CONF.image_feature_enabled.api_v1:
            image_list = glance_client.list_images(
                detail=True, **params)['images']
        else:
            image_list = glance_client.list_images(params)['images']
        self.assertEqual(2, len(image_list),
                         'Unexpected number of images for '
                         'v2:test_create_backup; was the oldest backup not '
                         'yet deleted? Image list: %s' %
                         [image['name'] for image in image_list])
        self.assertEqual((backup2, backup3),
                         (image_list[0]['name'], image_list[1]['name']))

    def _get_output(self):
        output = self.client.get_console_output(
            self.server_id, length=3)['output']
        self.assertTrue(output, "Console output was empty.")
        lines = len(output.split('\n'))
        self.assertEqual(lines, 3)

    @decorators.idempotent_id('4b8867e6-fffa-4d54-b1d1-6fdda57be2f3')
    @testtools.skipUnless(CONF.compute_feature_enabled.console_output,
                          'Console output not supported.')
    def test_get_console_output(self):
        """Test getting console output for a server

        Should be able to GET the console output for a given server_id and
        number of lines.
        """

        # This reboot is necessary for outputting some console log after
        # creating an instance backup. If an instance backup, the console
        # log file is truncated and we cannot get any console log through
        # "console-log" API.
        # The detail is https://bugs.launchpad.net/nova/+bug/1251920
        self.client.reboot_server(self.server_id, type='HARD')
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        self.wait_for(self._get_output)

    @decorators.idempotent_id('89104062-69d8-4b19-a71b-f47b7af093d7')
    @testtools.skipUnless(CONF.compute_feature_enabled.console_output,
                          'Console output not supported.')
    def test_get_console_output_with_unlimited_size(self):
        """Test getting server's console output with unlimited size

        The console output lines length should be bigger than the one
        of test_get_console_output.
        """
        server = self.create_test_server(wait_until='ACTIVE')

        def _check_full_length_console_log():
            output = self.client.get_console_output(server['id'])['output']
            self.assertTrue(output, "Console output was empty.")
            lines = len(output.split('\n'))

            # NOTE: This test tries to get full length console log, and the
            # length should be bigger than the one of test_get_console_output.
            self.assertGreater(lines, 3, "Cannot get enough console log "
                                         "length. (lines: %s)" % lines)

        self.wait_for(_check_full_length_console_log)

    @decorators.idempotent_id('5b65d4e7-4ecd-437c-83c0-d6b79d927568')
    @testtools.skipUnless(CONF.compute_feature_enabled.console_output,
                          'Console output not supported.')
    def test_get_console_output_server_id_in_shutoff_status(self):
        """Test getting console output for a server in SHUTOFF status

        Should be able to GET the console output for a given server_id
        in SHUTOFF status.
        """

        # NOTE: SHUTOFF is irregular status. To avoid test instability,
        #       one server is created only for this test without using
        #       the server that was created in setUpClass.
        server = self.create_test_server(wait_until='ACTIVE')
        temp_server_id = server['id']

        self.client.stop_server(temp_server_id)
        waiters.wait_for_server_status(self.client, temp_server_id, 'SHUTOFF')
        self.wait_for(self._get_output)

    @decorators.idempotent_id('bd61a9fd-062f-4670-972b-2d6c3e3b9e73')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    def test_pause_unpause_server(self):
        """Test pausing and unpausing server"""
        self.client.pause_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'PAUSED')
        self.client.unpause_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')

    @decorators.idempotent_id('0d8ee21e-b749-462d-83da-b85b41c86c7f')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    def test_suspend_resume_server(self):
        """Test suspending and resuming server"""
        self.client.suspend_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id,
                                       'SUSPENDED')
        self.client.resume_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')

    @decorators.idempotent_id('77eba8e0-036e-4635-944b-f7a8f3b78dc9')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    @utils.services('image')
    def test_shelve_unshelve_server(self):
        """Test shelving and unshelving server"""
        if CONF.image_feature_enabled.api_v2:
            glance_client = self.os_primary.image_client_v2
        elif CONF.image_feature_enabled.api_v1:
            glance_client = self.os_primary.image_client
        else:
            raise lib_exc.InvalidConfiguration(
                'Either api_v1 or api_v2 must be True in '
                '[image-feature-enabled].')
        compute.shelve_server(self.client, self.server_id,
                              force_shelve_offload=True)

        def _unshelve_server():
            server_info = self.client.show_server(self.server_id)['server']
            if 'SHELVED' in server_info['status']:
                self.client.unshelve_server(self.server_id)
        self.addCleanup(_unshelve_server)

        server = self.client.show_server(self.server_id)['server']
        image_name = server['name'] + '-shelved'
        params = {'name': image_name}
        if CONF.image_feature_enabled.api_v2:
            images = glance_client.list_images(params)['images']
        elif CONF.image_feature_enabled.api_v1:
            images = glance_client.list_images(
                detail=True, **params)['images']
        self.assertEqual(1, len(images))
        self.assertEqual(image_name, images[0]['name'])

        self.client.unshelve_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')
        glance_client.wait_for_resource_deletion(images[0]['id'])

    @decorators.idempotent_id('8cf9f450-a871-42cf-9bef-77eba189c0b0')
    @decorators.related_bug('1745529')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    def test_shelve_paused_server(self):
        """Test shelving a paused server"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.client.pause_server(server['id'])
        waiters.wait_for_server_status(self.client, server['id'], 'PAUSED')
        # Check if Shelve operation is successful on paused server.
        compute.shelve_server(self.client, server['id'],
                              force_shelve_offload=True)

    @decorators.idempotent_id('af8eafd4-38a7-4a4b-bdbc-75145a580560')
    def test_stop_start_server(self):
        """Test stopping and starting server"""
        self.client.stop_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'SHUTOFF')
        self.client.start_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')

    @decorators.idempotent_id('80a8094c-211e-440a-ab88-9e59d556c7ee')
    def test_lock_unlock_server(self):
        """Test locking and unlocking server

        Lock the server, and trying to stop it will fail because locked
        server is not allowed to be stopped by non-admin user.
        Then unlock the server, now the server can be stopped and started.
        """
        # Lock the server,try server stop(exceptions throw),unlock it and retry
        self.client.lock_server(self.server_id)
        self.addCleanup(self.client.unlock_server, self.server_id)
        server = self.client.show_server(self.server_id)['server']
        self.assertEqual(server['status'], 'ACTIVE')
        # Locked server is not allowed to be stopped by non-admin user
        self.assertRaises(lib_exc.Conflict,
                          self.client.stop_server, self.server_id)
        self.client.unlock_server(self.server_id)
        self.client.stop_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'SHUTOFF')
        self.client.start_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'ACTIVE')

    def _validate_url(self, url):
        valid_scheme = ['http', 'https']
        parsed_url = urlparse.urlparse(url)
        self.assertNotEqual('None', parsed_url.port)
        self.assertNotEqual('None', parsed_url.hostname)
        self.assertIn(parsed_url.scheme, valid_scheme)

    @decorators.idempotent_id('c6bc11bf-592e-4015-9319-1c98dc64daf5')
    @testtools.skipUnless(CONF.compute_feature_enabled.vnc_console,
                          'VNC Console feature is disabled.')
    def test_get_vnc_console(self):
        """Test getting vnc console from a server

        The returned vnc console url should be in valid format.
        """
        if self.is_requested_microversion_compatible('2.5'):
            body = self.client.get_vnc_console(
                self.server_id, type='novnc')['console']
        else:
            body = self.client.get_remote_console(
                self.server_id, console_type='novnc',
                protocol='vnc')['remote_console']
        self.assertEqual('novnc', body['type'])
        self.assertNotEqual('', body['url'])
        self._validate_url(body['url'])



# Copyright 2017 AT&T Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
https://docs.openstack.org/api-ref/identity/v3-ext/#os-ep-filter-api
"""

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class EndPointsFilterClient(rest_client.RestClient):
    api_version = "v3"
    ep_filter = "OS-EP-FILTER"

    def list_projects_for_endpoint(self, endpoint_id):
        """List all projects that are associated with the endpoint."""
        resp, body = self.get(self.ep_filter + '/endpoints/%s/projects' %
                              endpoint_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def add_endpoint_to_project(self, project_id, endpoint_id):
        """Add association between project and endpoint. """
        body = None
        resp, body = self.put(
            self.ep_filter + '/projects/%s/endpoints/%s' %
            (project_id, endpoint_id), body)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def check_endpoint_in_project(self, project_id, endpoint_id):
        """Check association of Project with Endpoint."""
        resp, body = self.head(
            self.ep_filter + '/projects/%s/endpoints/%s' %
            (project_id, endpoint_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_endpoints_in_project(self, project_id):
        """List Endpoints associated with Project."""
        resp, body = self.get(self.ep_filter + '/projects/%s/endpoints'
                              % project_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_endpoint_from_project(self, project_id, endpoint_id):
        """Delete association between project and endpoint."""
        resp, body = self.delete(
            self.ep_filter + '/projects/%s/endpoints/%s'
            % (project_id, endpoint_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_endpoint_groups_for_project(self, project_id):
        """List Endpoint Groups Associated with Project."""
        resp, body = self.get(
            self.ep_filter + '/projects/%s/endpoint_groups'
            % project_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_projects_for_endpoint_group(self, endpoint_group_id):
        """List Projects Associated with Endpoint Group."""
        resp, body = self.get(
            self.ep_filter + '/endpoint_groups/%s/projects'
            % endpoint_group_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_endpoints_for_endpoint_group(self, endpoint_group_id):
        """List Endpoints Associated with Endpoint Group."""
        resp, body = self.get(
            self.ep_filter + '/endpoint_groups/%s/endpoints'
            % endpoint_group_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def add_endpoint_group_to_project(self, endpoint_group_id, project_id):
        """Create Endpoint Group to Project Association."""
        body = None
        resp, body = self.put(
            self.ep_filter + '/endpoint_groups/%s/projects/%s'
            % (endpoint_group_id, project_id), body)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_endpoint_group_for_project(self, endpoint_group_id, project_id):
        """Get Endpoint Group to Project Association."""
        resp, body = self.get(
            self.ep_filter + '/endpoint_groups/%s/projects/%s'
            % (endpoint_group_id, project_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_endpoint_group_from_project(
        self, endpoint_group_id, project_id):
        """Delete Endpoint Group to Project Association."""
        resp, body = self.delete(
            self.ep_filter + '/endpoint_groups/%s/projects/%s'
            % (endpoint_group_id, project_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 Hewlett Packard Enterprise Development LP
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from tempest.lib.common import cred_client
from tempest.tests import base


class TestCredClientV2(base.TestCase):
    def setUp(self):
        super(TestCredClientV2, self).setUp()
        self.identity_client = mock.MagicMock()
        self.projects_client = mock.MagicMock()
        self.users_client = mock.MagicMock()
        self.roles_client = mock.MagicMock()
        self.creds_client = cred_client.V2CredsClient(self.identity_client,
                                                      self.projects_client,
                                                      self.users_client,
                                                      self.roles_client)

    def test_create_project(self):
        self.projects_client.create_tenant.return_value = {
            'tenant': 'a_tenant'
        }
        res = self.creds_client.create_project('fake_name', 'desc')
        self.assertEqual('a_tenant', res)
        self.projects_client.create_tenant.assert_called_once_with(
            name='fake_name', description='desc')

    def test_delete_project(self):
        self.creds_client.delete_project('fake_id')
        self.projects_client.delete_tenant.assert_called_once_with(
            'fake_id')

    def test_get_credentials(self):
        ret = self.creds_client.get_credentials(
            {'name': 'some_user', 'id': 'fake_id'},
            {'name': 'some_project', 'id': 'fake_id'},
            'password123')
        self.assertEqual(ret.username, 'some_user')
        self.assertEqual(ret.project_name, 'some_project')


class TestCredClientV3(base.TestCase):
    def setUp(self):
        super(TestCredClientV3, self).setUp()
        self.identity_client = mock.MagicMock()
        self.projects_client = mock.MagicMock()
        self.users_client = mock.MagicMock()
        self.roles_client = mock.MagicMock()
        self.domains_client = mock.MagicMock()
        self.domains_client.list_domains.return_value = {
            'domains': [{'id': 'fake_domain_id', 'name': 'some_domain'}]
        }
        self.creds_client = cred_client.V3CredsClient(self.identity_client,
                                                      self.projects_client,
                                                      self.users_client,
                                                      self.roles_client,
                                                      self.domains_client,
                                                      'fake_domain')

    def test_create_project(self):
        self.projects_client.create_project.return_value = {
            'project': 'a_tenant'
        }
        res = self.creds_client.create_project('fake_name', 'desc')
        self.assertEqual('a_tenant', res)
        self.projects_client.create_project.assert_called_once_with(
            name='fake_name', description='desc', domain_id='fake_domain_id')

    def test_delete_project(self):
        self.creds_client.delete_project('fake_id')
        self.projects_client.delete_project.assert_called_once_with(
            'fake_id')

    def test_get_credentials(self):
        ret = self.creds_client.get_credentials(
            {'name': 'some_user', 'id': 'fake_id'},
            {'name': 'some_project', 'id': 'fake_id'},
            'password123')
        self.assertEqual(ret.username, 'some_user')
        self.assertEqual(ret.project_name, 'some_project')
        self.assertIsNone(ret.system)
        self.assertEqual(ret.domain_name, 'some_domain')
        ret = self.creds_client.get_credentials(
            {'name': 'some_user', 'id': 'fake_id'},
            None,
            'password123',
            domain={'name': 'another_domain', 'id': 'another_id'})
        self.assertEqual(ret.username, 'some_user')
        self.assertIsNone(ret.project_name)
        self.assertIsNone(ret.system)
        self.assertEqual(ret.domain_name, 'another_domain')
        ret = self.creds_client.get_credentials(
            {'name': 'some_user', 'id': 'fake_id'},
            None,
            'password123',
            system={'system': 'all'})
        self.assertEqual(ret.username, 'some_user')
        self.assertIsNone(ret.project_name)
        self.assertEqual(ret.system, {'system': 'all'})
        self.assertEqual(ret.domain_name, 'some_domain')

    def test_create_user(self):
        self.users_client.create_user.return_value = {
            'user': 'a_user'
        }
        fake_project = {
            'id': 'fake_project_id',
        }
        res = self.creds_client.create_user('fake_username',
                                            'fake_password',
                                            fake_project,
                                            'fake_email')
        self.assertEqual('a_user', res)
        self.users_client.create_user.assert_called_once_with(
            name='fake_username', password='fake_password',
            project_id=fake_project['id'],
            email='fake_email',
            domain_id='fake_domain_id')



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class TokensV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test tokens"""

    credentials = ['primary', 'admin', 'alt']

    @decorators.idempotent_id('565fa210-1da1-4563-999b-f7b5b67cf112')
    def test_rescope_token(self):
        """Rescope a token.

        An unscoped token can be requested, that token can be used to request a
        scoped token. The scoped token can be revoked, and the original token
        used to get a token in a different project.

        """

        # Create a user.
        user_password = data_utils.rand_password()
        user = self.create_test_user(password=user_password)

        # Create a couple projects
        project1_name = data_utils.rand_name(name=self.__class__.__name__)
        project1 = self.setup_test_project(name=project1_name)

        project2_name = data_utils.rand_name(name=self.__class__.__name__)
        project2 = self.setup_test_project(name=project2_name)
        self.addCleanup(self.projects_client.delete_project, project2['id'])

        # Create a role
        role = self.setup_test_role()

        # Grant the user the role on both projects.
        self.roles_client.create_user_role_on_project(project1['id'],
                                                      user['id'],
                                                      role['id'])

        self.roles_client.create_user_role_on_project(project2['id'],
                                                      user['id'],
                                                      role['id'])

        # Get an unscoped token.
        token_auth = self.token.auth(user_id=user['id'],
                                     password=user_password)

        token_id = token_auth.response['x-subject-token']
        orig_expires_at = token_auth['token']['expires_at']
        orig_user = token_auth['token']['user']

        self.assertIsInstance(token_auth['token']['expires_at'], str)
        self.assertIsInstance(token_auth['token']['issued_at'], str)
        self.assertEqual(['password'], token_auth['token']['methods'])
        self.assertEqual(user['id'], token_auth['token']['user']['id'])
        self.assertEqual(user['name'], token_auth['token']['user']['name'])
        self.assertEqual(CONF.identity.default_domain_id,
                         token_auth['token']['user']['domain']['id'])
        self.assertIsNotNone(token_auth['token']['user']['domain']['name'])
        self.assertNotIn('catalog', token_auth['token'])
        self.assertNotIn('project', token_auth['token'])
        self.assertNotIn('roles', token_auth['token'])

        # Use the unscoped token to get a scoped token.
        token_auth = self.token.auth(
            token=token_id,
            project_name=project1_name,
            project_domain_id=CONF.identity.default_domain_id)
        token1_id = token_auth.response['x-subject-token']

        self.assertEqual(orig_expires_at, token_auth['token']['expires_at'],
                         'Expiration time should match original token')
        self.assertIsInstance(token_auth['token']['issued_at'], str)
        self.assertEqual(set(['password', 'token']),
                         set(token_auth['token']['methods']))
        self.assertEqual(orig_user, token_auth['token']['user'],
                         'User should match original token')
        self.assertIsInstance(token_auth['token']['catalog'], list)
        self.assertEqual(project1['id'],
                         token_auth['token']['project']['id'])
        self.assertEqual(project1['name'],
                         token_auth['token']['project']['name'])
        self.assertEqual(CONF.identity.default_domain_id,
                         token_auth['token']['project']['domain']['id'])
        self.assertIsNotNone(token_auth['token']['project']['domain']['name'])
        self.assertEqual(1, len(token_auth['token']['roles']))
        self.assertEqual(role['id'], token_auth['token']['roles'][0]['id'])
        self.assertEqual(role['name'], token_auth['token']['roles'][0]['name'])

        # Revoke the unscoped token.
        self.client.delete_token(token1_id)

        # Now get another scoped token using the unscoped token.
        token_auth = self.token.auth(
            token=token_id,
            project_name=project2_name,
            project_domain_id=CONF.identity.default_domain_id)

        self.assertEqual(project2['id'],
                         token_auth['token']['project']['id'])
        self.assertEqual(project2['name'],
                         token_auth['token']['project']['name'])

    @decorators.idempotent_id('08ed85ce-2ba8-4864-b442-bcc61f16ae89')
    def test_get_available_project_scopes(self):
        """Test getting available project scopes"""
        manager_project_id = self.os_primary.credentials.project_id
        admin_user_id = self.os_admin.credentials.user_id
        admin_role_id = self.get_role_by_name(CONF.identity.admin_role)['id']

        # Grant the user the role on both projects.
        self.roles_client.create_user_role_on_project(
            manager_project_id, admin_user_id, admin_role_id)
        self.addCleanup(
            self.roles_client.delete_role_from_user_on_project,
            manager_project_id, admin_user_id, admin_role_id)

        assigned_project_ids = [self.os_admin.credentials.project_id,
                                manager_project_id]

        # Get available project scopes
        available_projects = self.client.list_auth_projects()['projects']

        # Create list to save fetched project IDs
        fetched_project_ids = [i['id'] for i in available_projects]

        # verifying the project ids in list
        missing_project_ids = \
            [p for p in assigned_project_ids if p not in fetched_project_ids]
        self.assertEmpty(missing_project_ids,
                         "Failed to find project_ids %s in fetched list" %
                         ', '.join(missing_project_ids))

    @decorators.idempotent_id('ec5ecb05-af64-4c04-ac86-4d9f6f12f185')
    def test_get_available_domain_scopes(self):
        """Test getting available domain scopes

        To verify that listing domain scopes for a user works if
        the user has a domain role or belongs to a group that has a domain
        role. For this test, admin client is used to add roles to alt user,
        which performs API calls, to avoid 401 Unauthorized errors.
        """
        alt_user_id = self.os_alt.credentials.user_id

        def _create_user_domain_role_for_alt_user():
            domain_id = self.setup_test_domain()['id']
            role_id = self.setup_test_role()['id']

            # Create a role association between the user and domain.
            self.roles_client.create_user_role_on_domain(
                domain_id, alt_user_id, role_id)
            self.addCleanup(
                self.roles_client.delete_role_from_user_on_domain,
                domain_id, alt_user_id, role_id)

            return domain_id

        def _create_group_domain_role_for_alt_user():
            domain_id = self.setup_test_domain()['id']
            role_id = self.setup_test_role()['id']

            # Create a group.
            group_id = self.setup_test_group(domain_id=domain_id)['id']

            # Add the alt user to the group.
            self.groups_client.add_group_user(group_id, alt_user_id)
            self.addCleanup(self.groups_client.delete_group_user,
                            group_id, alt_user_id)

            # Create a role association between the group and domain.
            self.roles_client.create_group_role_on_domain(
                domain_id, group_id, role_id)
            self.addCleanup(
                self.roles_client.delete_role_from_group_on_domain,
                domain_id, group_id, role_id)

            return domain_id

        # Add the alt user to 2 random domains and 2 random groups
        # with randomized domains and roles.
        assigned_domain_ids = []
        for _ in range(2):
            domain_id = _create_user_domain_role_for_alt_user()
            assigned_domain_ids.append(domain_id)
            domain_id = _create_group_domain_role_for_alt_user()
            assigned_domain_ids.append(domain_id)

        # Get available domain scopes for the alt user.
        available_domains = self.os_alt.identity_v3_client.list_auth_domains()[
            'domains']
        fetched_domain_ids = [i['id'] for i in available_domains]

        # Verify the expected domain IDs are in the list.
        missing_domain_ids = \
            [p for p in assigned_domain_ids if p not in fetched_domain_ids]
        self.assertEmpty(missing_domain_ids,
                         "Failed to find domain_ids %s in fetched list"
                         % ", ".join(missing_domain_ids))



#!/usr/bin/env python2

# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Track test skips via launchpadlib API and raise alerts if a bug
is fixed but a skip is still in the Tempest test code
"""

import argparse
import os
import re

from oslo_log import log as logging

try:
    from launchpadlib import launchpad
except ImportError:
    launchpad = None

LPCACHEDIR = os.path.expanduser(os.path.join('~', '.launchpadlib', 'cache'))
LOG = logging.getLogger(__name__)

BASEDIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                       '..', '..', '..'))
TESTDIR = os.path.join(BASEDIR, 'tempest')


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('test_path', nargs='?', default=TESTDIR,
                        help='Path of test dir')
    return parser.parse_args()


def info(msg, *args, **kwargs):
    LOG.info(msg, *args, **kwargs)


def debug(msg, *args, **kwargs):
    LOG.debug(msg, *args, **kwargs)


def find_skips(start):
    """Find the entire list of skipped tests.

    Returns a list of tuples (method, bug) that represent
    test methods that have been decorated to skip because of
    a particular bug.
    """
    results = {}
    debug("Searching in %s", start)
    for root, _dirs, files in os.walk(start):
        for name in files:
            if name.startswith('test_') and name.endswith('py'):
                path = os.path.join(root, name)
                debug("Searching in %s", path)
                temp_result = find_skips_in_file(path)
                for method_name, bug_no in temp_result:
                    if results.get(bug_no):
                        result_dict = results.get(bug_no)
                        if result_dict.get(name):
                            result_dict[name].append(method_name)
                        else:
                            result_dict[name] = [method_name]
                        results[bug_no] = result_dict
                    else:
                        results[bug_no] = {name: [method_name]}
    return results


def find_skips_in_file(path):
    """Return the skip tuples in a test file."""
    BUG_RE = re.compile(r'\s*@.*skip_because\(bug=[\'"](\d+)[\'"]')
    DEF_RE = re.compile(r'\s*def (\w+)\(')
    bug_found = False
    results = []
    with open(path, 'rb') as content:
        lines = content.readlines()
        for x, line in enumerate(lines):
            if not bug_found:
                res = BUG_RE.match(line)
                if res:
                    bug_no = int(res.group(1))
                    debug("Found bug skip %s on line %d", bug_no, x + 1)
                    bug_found = True
            else:
                res = DEF_RE.match(line)
                if res:
                    method = res.group(1)
                    debug("Found test method %s skips for bug %d",
                          method, bug_no)
                    results.append((method, bug_no))
                    bug_found = False
    return results


def get_results(result_dict):
    results = []
    for bug_no in result_dict:
        for method in result_dict[bug_no]:
            results.append((method, bug_no))
    return results


def main():
    parser = parse_args()
    results = find_skips(parser.test_path)
    unique_bugs = sorted(set([bug for (method, bug) in get_results(results)]))
    unskips = []
    duplicates = []
    info("Total bug skips found: %d", len(results))
    info("Total unique bugs causing skips: %d", len(unique_bugs))
    if launchpad is not None:
        lp = launchpad.Launchpad.login_anonymously('grabbing bugs',
                                                   'production',
                                                   LPCACHEDIR)
    else:
        print("To check the bug status launchpadlib should be installed")
        exit(1)

    for bug_no in unique_bugs:
        bug = lp.bugs[bug_no]
        duplicate = bug.duplicate_of_link
        if duplicate is not None:
            dup_id = duplicate.split('/')[-1]
            duplicates.append((bug_no, dup_id))
        for task in bug.bug_tasks:
            info("Bug #%7s (%12s - %12s)", bug_no,
                 task.importance, task.status)
            if task.status in ('Fix Released', 'Fix Committed'):
                unskips.append(bug_no)

    for bug_id, dup_id in duplicates:
        if bug_id not in unskips:
            dup_bug = lp.bugs[dup_id]
            for task in dup_bug.bug_tasks:
                info("Bug #%7s is a duplicate of Bug#%7s (%12s - %12s)",
                     bug_id, dup_id, task.importance, task.status)
                if task.status in ('Fix Released', 'Fix Committed'):
                    unskips.append(bug_id)

    unskips = sorted(set(unskips))
    if unskips:
        print("The following bugs have been fixed and the corresponding skips")
        print("should be removed from the test cases:")
        print()
        for bug in unskips:
            message = "  %7s in " % bug
            locations = ["%s" % x for x in results[bug].keys()]
            message += " and ".join(locations)
            print(message)


if __name__ == '__main__':
    main()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.lib.services.network import tags_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestTagsClient(base.BaseServiceTest):

    FAKE_TAGS = {
        "tags": [
            "red",
            "blue"
        ]
    }

    FAKE_RESOURCE_TYPE = 'network'

    FAKE_RESOURCE_ID = '7a8f904b-c1ed-4446-a87d-60440c02934b'

    def setUp(self):
        super(TestTagsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = tags_client.TagsClient(
            fake_auth, 'network', 'regionOne')

    def _test_update_all_tags(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_all_tags,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_TAGS,
            bytes_body,
            resource_type=self.FAKE_RESOURCE_TYPE,
            resource_id=self.FAKE_RESOURCE_ID,
            tags=self.FAKE_TAGS)

    def _test_check_tag_existence(self, bytes_body=False):
        self.check_service_client_function(
            self.client.check_tag_existence,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            bytes_body,
            resource_type=self.FAKE_RESOURCE_TYPE,
            resource_id=self.FAKE_RESOURCE_ID,
            tag=self.FAKE_TAGS['tags'][0],
            status=204)

    def _test_create_tag(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_tag,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            resource_type=self.FAKE_RESOURCE_TYPE,
            resource_id=self.FAKE_RESOURCE_ID,
            tag=self.FAKE_TAGS['tags'][0],
            status=201)

    def _test_list_tags(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_tags,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_TAGS,
            bytes_body,
            resource_type=self.FAKE_RESOURCE_TYPE,
            resource_id=self.FAKE_RESOURCE_ID)

    def test_update_all_tags_with_str_body(self):
        self._test_update_all_tags()

    def test_update_all_tags_with_bytes_body(self):
        self._test_update_all_tags(bytes_body=True)

    def test_delete_all_tags(self):
        self.check_service_client_function(
            self.client.delete_all_tags,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            resource_type=self.FAKE_RESOURCE_TYPE,
            resource_id=self.FAKE_RESOURCE_ID,
            status=204)

    def test_check_tag_existence_with_str_body(self):
        self._test_check_tag_existence()

    def test_check_tag_existence_with_bytes_body(self):
        self._test_check_tag_existence(bytes_body=True)

    def test_create_tag_with_str_body(self):
        self._test_create_tag()

    def test_create_tag_with_bytes_body(self):
        self._test_create_tag(bytes_body=True)

    def test_list_tags_with_str_body(self):
        self._test_list_tags()

    def test_list_tags_with_bytes_body(self):
        self._test_list_tags(bytes_body=True)

    def test_delete_tag(self):
        self.check_service_client_function(
            self.client.delete_tag,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            resource_type=self.FAKE_RESOURCE_TYPE,
            resource_id=self.FAKE_RESOURCE_ID,
            tag=self.FAKE_TAGS['tags'][0],
            status=204)



# Copyright 2015 Dell Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from cliff import app
from cliff import commandmanager
from oslo_log import log as logging
from pbr import version


class Main(app.App):

    log = logging.getLogger(__name__)

    def __init__(self):
        super(Main, self).__init__(
            description='Tempest cli application',
            version=version.VersionInfo('tempest').version_string_with_vcs(),
            command_manager=commandmanager.CommandManager('tempest.cm'),
            deferred_help=True,
            )

    def initialize_app(self, argv):
        self.log.debug('tempest initialize_app')

    def prepare_to_run_command(self, cmd):
        self.log.debug('prepare_to_run_command %s', cmd.__class__.__name__)

    def clean_up(self, cmd, result, err):
        self.log.debug('tempest clean_up %s', cmd.__class__.__name__)
        if err:
            self.log.debug('tempest got an error: %s', err)


def main(argv=sys.argv[1:]):
    the_app = Main()
    return the_app.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v1 import snapshots_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSnapshotsClient(base.BaseServiceTest):
    FAKE_CREATE_SNAPSHOT = {
        "snapshot": {
            "display_name": "snap-001",
            "display_description": "Daily backup",
            "volume_id": "521752a6-acf6-4b2d-bc7a-119f9148cd8c",
            "force": True
        }
    }

    FAKE_UPDATE_SNAPSHOT_REQUEST = {
        "metadata": {
            "key": "v1"
        }
    }

    FAKE_INFO_SNAPSHOT = {
        "snapshot": {
            "id": "3fbbcccf-d058-4502-8844-6feeffdf4cb5",
            "display_name": "snap-001",
            "display_description": "Daily backup",
            "volume_id": "521752a6-acf6-4b2d-bc7a-119f9148cd8c",
            "status": "available",
            "size": 30,
            "created_at": "2012-02-29T03:50:07Z"
        }
    }

    FAKE_LIST_SNAPSHOTS = {
        "snapshots": [
            {
                "id": "3fbbcccf-d058-4502-8844-6feeffdf4cb5",
                "display_name": "snap-001",
                "display_description": "Daily backup",
                "volume_id": "521752a6-acf6-4b2d-bc7a-119f9148cd8c",
                "status": "available",
                "size": 30,
                "created_at": "2012-02-29T03:50:07Z",
                "metadata": {
                    "contents": "junk"
                }
            },
            {
                "id": "e479997c-650b-40a4-9dfe-77655818b0d2",
                "display_name": "snap-002",
                "display_description": "Weekly backup",
                "volume_id": "76b8950a-8594-4e5b-8dce-0dfa9c696358",
                "status": "available",
                "size": 25,
                "created_at": "2012-03-19T01:52:47Z",
                "metadata": {}
            }
        ]
    }

    def setUp(self):
        super(TestSnapshotsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = snapshots_client.SnapshotsClient(fake_auth,
                                                       'volume',
                                                       'regionOne')

    def _test_create_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SNAPSHOT,
            bytes_body)

    def _test_show_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_snapshot,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_SNAPSHOT,
            bytes_body,
            snapshot_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_list_snapshots(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_snapshots,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_SNAPSHOTS,
            bytes_body,
            detail=True)

    def _test_create_snapshot_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_snapshot_metadata,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_INFO_SNAPSHOT,
            bytes_body,
            snapshot_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5",
            metadata={"key": "v1"})

    def _test_update_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_snapshot,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_SNAPSHOT_REQUEST,
            bytes_body,
            snapshot_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_show_snapshot_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_snapshot_metadata,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_UPDATE_SNAPSHOT_REQUEST,
            bytes_body,
            snapshot_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_update_snapshot_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_snapshot_metadata,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_SNAPSHOT_REQUEST,
            bytes_body, snapshot_id="cbc36478b0bd8e67e89")

    def _test_update_snapshot_metadata_item(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_snapshot_metadata_item,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_INFO_SNAPSHOT,
            bytes_body, volume_type_id="cbc36478b0bd8e67e89")

    def test_create_snapshot_with_str_body(self):
        self._test_create_snapshot()

    def test_create_snapshot_with_bytes_body(self):
        self._test_create_snapshot(bytes_body=True)

    def test_show_snapshot_with_str_body(self):
        self._test_show_snapshot()

    def test_show_snapshot_with_bytes_body(self):
        self._test_show_snapshot(bytes_body=True)

    def test_list_snapshots_with_str_body(self):
        self._test_list_snapshots()

    def test_list_snapshots_with_bytes_body(self):
        self._test_list_snapshots(bytes_body=True)

    def test_create_snapshot_metadata_with_str_body(self):
        self._test_create_snapshot_metadata()

    def test_create_snapshot_metadata_with_bytes_body(self):
        self._test_create_snapshot_metadata(bytes_body=True)

    def test_update_snapshot_with_str_body(self):
        self._test_update_snapshot()

    def test_update_snapshot_with_bytes_body(self):
        self._test_update_snapshot(bytes_body=True)

    def test_show_snapshot_metadata_with_str_body(self):
        self._test_show_snapshot_metadata()

    def test_show_snapshot_metadata_with_bytes_body(self):
        self._test_show_snapshot_metadata(bytes_body=True)

    def test_update_snapshot_metadata_with_str_body(self):
        self._test_update_snapshot_metadata()

    def test_update_snapshot_metadata_with_bytes_body(self):
        self._test_update_snapshot_metadata(bytes_body=True)

    def test_force_delete_snapshot(self):
        self.check_service_client_function(
            self.client.force_delete_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            snapshot_id="521752a6-acf6-4b2d-bc7a-119f9148cd8c",
            status=202)

    def test_delete_snapshot(self):
        self.check_service_client_function(
            self.client.delete_snapshot,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            snapshot_id="521752a6-acf6-4b2d-bc7a-119f9148cd8c",
            status=202)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy

from tempest.lib.api_schema.response.compute.v2_1 import interfaces

# ****** Schemas changed in microversion 2.70 *****************
#
# 1. add optional field 'tag' in the Response body of the following APIs:
#    - GET /servers/{server_id}/os-interface
#    - POST /servers/{server_id}/os-interface
#    - GET /servers/{server_id}/os-interface/{port_id}

get_create_interfaces = copy.deepcopy(interfaces.get_create_interfaces)
get_create_interfaces['response_body']['properties']['interfaceAttachment'][
    'properties'].update({'tag': {'type': ['string', 'null']}})

list_interfaces = copy.deepcopy(interfaces.list_interfaces)
list_interfaces['response_body']['properties']['interfaceAttachments'][
    'items']['properties'].update({'tag': {'type': ['string', 'null']}})

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.1 ***
delete_interface = copy.deepcopy(interfaces.delete_interface)



# Copyright 2015 Deutsche Telekom AG.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
from oslo_serialization import jsonutils as json

from tempest.tests import base
from tempest.tests.lib import fake_http


class BaseServiceTest(base.TestCase):
    def create_response(self, body, to_utf=False, status=200, headers=None):
        json_body = {}
        if body:
            json_body = json.dumps(body)
            if to_utf:
                json_body = json_body.encode('utf-8')
        resp = fake_http.fake_http_response(headers, status=status), json_body
        return resp

    def check_service_client_function(self, function, function2mock,
                                      body, to_utf=False, status=200,
                                      headers=None, mock_args=None,
                                      resp_as_string=False,
                                      **kwargs):
        """Mock a service client function for unit testing.

        :param function: The service client function to call.
        :param function2mock: The REST call to mock inside the service client
               function.
        :param body: Expected response body returned by the service client
               function.
        :param to_utf: Whether to use UTF-8 encoding for response.
        :param status: Expected response status returned by the service client
               function.
        :param headers: Expected headers returned by the service client
               function.
        :param mock_args: List/dict/value of expected args/kwargs called by
               function2mock. For example:
               * If mock_args=['foo'] then ``assert_called_once_with('foo')``
                 is called.
               * If mock_args={'foo': 'bar'} then
                 ``assert_called_once_with(foo='bar')`` is called.
               * If mock_args='foo' then ``assert_called_once_with('foo')``
                 is called.
        :param resp_as_string: Whether response body is retruned as string.
               This is for service client methods which return ResponseBodyData
               object.
        :param kwargs: kwargs that are passed to function.
        """
        mocked_response = self.create_response(body, to_utf, status, headers)
        fixture = self.useFixture(fixtures.MockPatch(
            function2mock, return_value=mocked_response))
        if kwargs:
            resp = function(**kwargs)
        else:
            resp = function()
        if resp_as_string:
            resp = resp.data
        self.assertEqual(body, resp)
        if isinstance(mock_args, list):
            fixture.mock.assert_called_once_with(*mock_args)
        elif isinstance(mock_args, dict):
            fixture.mock.assert_called_once_with(**mock_args)
        elif mock_args is not None:
            fixture.mock.assert_called_once_with(mock_args)



# Copyright (c) 2019 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.placement import placement_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestPlacementClient(base.BaseServiceTest):
    FAKE_ALLOCATION_CANDIDATES = {
        'allocation_requests': [
            {'allocations': {
                'rp-uuid': {'resources': {'VCPU': 42}}
            }}
        ],
        'provider_summaries': {
            'rp-uuid': {
                'resources': {
                    'VCPU': {'used': 0, 'capacity': 64},
                    'MEMORY_MB': {'capacity': 11196, 'used': 0},
                    'DISK_GB': {'capacity': 19, 'used': 0}
                },
                'traits': ["HW_CPU_X86_SVM"],
            }
        }
    }

    FAKE_ALLOCATIONS = {
        'allocations': {
            'rp-uuid-1': {
                'resources': {
                    'NET_BW_IGR_KILOBIT_PER_SEC': 1
                },
                'generation': 14
            },
            'rp-uuid2': {
                'resources': {
                    'MEMORY_MB': 256,
                    'VCPU': 1
                },
                'generation': 9
            }
        }
    }

    def setUp(self):
        super(TestPlacementClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = placement_client.PlacementClient(
            fake_auth, 'placement', 'regionOne')

    def _test_list_allocation_candidates(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_allocation_candidates,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ALLOCATION_CANDIDATES,
            to_utf=bytes_body,
            **{'resources1': 'NET_BW_IGR_KILOBIT_PER_SEC:1'})

    def test_list_allocation_candidates_with_str_body(self):
        self._test_list_allocation_candidates()

    def test_list_allocation_candidates_with_bytes_body(self):
        self._test_list_allocation_candidates(bytes_body=True)

    def _test_list_allocations(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_allocations,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ALLOCATIONS,
            to_utf=bytes_body,
            **{'consumer_uuid': 'foo-bar'})

    def test_list_allocations_with_str_body(self):
        self._test_list_allocations()

    def test_list_allocations_with_bytes_body(self):
        self._test_list_allocations(bytes_body=True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
import testtools

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class RoutersTest(base.BaseNetworkTest):
    """Test routers"""

    def _add_router_interface_with_subnet_id(self, router_id, subnet_id):
        interface = self.routers_client.add_router_interface(
            router_id, subnet_id=subnet_id)
        self.addCleanup(self._remove_router_interface_with_subnet_id,
                        router_id, subnet_id)
        self.assertEqual(subnet_id, interface['subnet_id'])
        return interface

    def _remove_router_interface_with_subnet_id(self, router_id, subnet_id):
        body = self.routers_client.remove_router_interface(router_id,
                                                           subnet_id=subnet_id)
        self.assertEqual(subnet_id, body['subnet_id'])

    @classmethod
    def skip_checks(cls):
        super(RoutersTest, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('f64403e2-8483-4b34-8ccd-b09a87bcc68c')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_create_show_list_update_delete_router(self):
        """Test create/show/list/update/delete of a router"""
        # Create a router
        router_name = data_utils.rand_name(self.__class__.__name__ + '-router')
        router = self.create_router(
            router_name,
            admin_state_up=False,
            external_network_id=CONF.network.public_network_id)
        self.addCleanup(self.delete_router, router)
        self.assertEqual(router['name'], router_name)
        self.assertEqual(router['admin_state_up'], False)
        self.assertEqual(
            router['external_gateway_info']['network_id'],
            CONF.network.public_network_id)
        # Show details of the created router
        router_show = self.routers_client.show_router(
            router['id'])['router']
        self.assertEqual(router_show['name'], router['name'])
        self.assertEqual(
            router_show['external_gateway_info']['network_id'],
            CONF.network.public_network_id)
        # List routers and verify if created router is there in response
        routers = self.routers_client.list_routers()['routers']
        self.assertIn(router['id'], map(lambda x: x['id'], routers))
        # Update the name of router and verify if it is updated
        updated_name = 'updated' + router['name']
        router_update = self.routers_client.update_router(
            router['id'], name=updated_name)['router']
        self.assertEqual(router_update['name'], updated_name)
        router_show = self.routers_client.show_router(
            router['id'])['router']
        self.assertEqual(router_show['name'], updated_name)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('b42e6e39-2e37-49cc-a6f4-8467e940900a')
    def test_add_remove_router_interface_with_subnet_id(self):
        """Test adding and removing router interface with subnet id"""
        network_name = data_utils.rand_name(self.__class__.__name__)
        network = self.networks_client.create_network(
            name=network_name)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network, network['id'])
        subnet = self.create_subnet(network)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnets_client.delete_subnet, subnet['id'])
        router = self.create_router()
        self.addCleanup(self.delete_router, router)
        # Add router interface with subnet id
        interface = self.routers_client.add_router_interface(
            router['id'], subnet_id=subnet['id'])
        self.addCleanup(self._remove_router_interface_with_subnet_id,
                        router['id'], subnet['id'])
        self.assertIn('subnet_id', interface.keys())
        self.assertIn('port_id', interface.keys())
        # Verify router id is equal to device id in port details
        show_port_body = self.ports_client.show_port(
            interface['port_id'])
        self.assertEqual(show_port_body['port']['device_id'],
                         router['id'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2b7d2f37-6748-4d78-92e5-1d590234f0d5')
    def test_add_remove_router_interface_with_port_id(self):
        """Test adding and removing router interface with port id"""
        network_name = data_utils.rand_name(self.__class__.__name__)
        network = self.networks_client.create_network(
            name=network_name)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network, network['id'])
        subnet = self.create_subnet(network)
        self.addCleanup(self.subnets_client.delete_subnet, subnet['id'])
        router = self.create_router()
        self.addCleanup(self.delete_router, router)
        port_body = self.ports_client.create_port(
            network_id=network['id'],
            name=data_utils.rand_name(self.__class__.__name__))
        # add router interface to port created above
        interface = self.routers_client.add_router_interface(
            router['id'],
            port_id=port_body['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.remove_router_interface,
                        router['id'], port_id=port_body['port']['id'])
        self.assertIn('subnet_id', interface.keys())
        self.assertIn('port_id', interface.keys())
        # Verify router id is equal to device id in port details
        show_port_body = self.ports_client.show_port(
            interface['port_id'])
        self.assertEqual(show_port_body['port']['device_id'],
                         router['id'])
        self.routers_client.remove_router_interface(
            router['id'], port_id=port_body['port']['id'])

    @decorators.idempotent_id('c86ac3a8-50bd-4b00-a6b8-62af84a0765c')
    @utils.requires_ext(extension='extraroute', service='network')
    def test_update_delete_extra_route(self):
        """Test updating and deleting router with extra route"""
        # Create different cidr for each subnet to avoid cidr duplicate
        # The cidr starts from project_cidr
        next_cidr = self.cidr
        # Prepare to build several routes
        test_routes = []
        routes_num = 4
        # Create a router
        router = self.create_router(admin_state_up=True)
        self.addCleanup(self.delete_router, router)
        self.addCleanup(
            self._delete_extra_routes,
            router['id'])
        # Update router extra route, second ip of the range is
        # used as next hop
        for i in range(routes_num):
            network_name = data_utils.rand_name(self.__class__.__name__)
            network = self.networks_client.create_network(
                name=network_name)['network']
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.networks_client.delete_network,
                            network['id'])
            subnet = self.create_subnet(network, cidr=next_cidr)
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.subnets_client.delete_subnet, subnet['id'])
            next_cidr = next_cidr.next()

            # Add router interface with subnet id
            self.create_router_interface(router['id'], subnet['id'])
            self.addCleanup(self._remove_router_interface_with_subnet_id,
                            router['id'], subnet['id'])
            cidr = netaddr.IPNetwork(subnet['cidr'])
            next_hop = str(cidr[2])
            destination = str(subnet['cidr'])
            test_routes.append(
                {'nexthop': next_hop, 'destination': destination}
            )

        test_routes.sort(key=lambda x: x['destination'])
        extra_route = self.routers_client.update_router(
            router['id'], routes=test_routes)
        show_body = self.routers_client.show_router(router['id'])
        # Assert the number of routes
        self.assertEqual(routes_num, len(extra_route['router']['routes']))
        self.assertEqual(routes_num, len(show_body['router']['routes']))

        routes = extra_route['router']['routes']
        routes.sort(key=lambda x: x['destination'])
        # Assert the nexthops & destination
        for i in range(routes_num):
            self.assertEqual(test_routes[i]['destination'],
                             routes[i]['destination'])
            self.assertEqual(test_routes[i]['nexthop'], routes[i]['nexthop'])

        routes = show_body['router']['routes']
        routes.sort(key=lambda x: x['destination'])
        for i in range(routes_num):
            self.assertEqual(test_routes[i]['destination'],
                             routes[i]['destination'])
            self.assertEqual(test_routes[i]['nexthop'], routes[i]['nexthop'])

        self._delete_extra_routes(router['id'])
        show_body_after_deletion = self.routers_client.show_router(
            router['id'])
        self.assertEmpty(show_body_after_deletion['router']['routes'])

    def _delete_extra_routes(self, router_id):
        self.routers_client.update_router(router_id, routes=None)

    @decorators.idempotent_id('a8902683-c788-4246-95c7-ad9c6d63a4d9')
    def test_update_router_admin_state(self):
        """Test updating router's admin state"""
        router = self.create_router()
        self.addCleanup(self.delete_router, router)
        self.assertFalse(router['admin_state_up'])
        # Update router admin state
        update_body = self.routers_client.update_router(router['id'],
                                                        admin_state_up=True)
        self.assertTrue(update_body['router']['admin_state_up'])
        show_body = self.routers_client.show_router(router['id'])
        self.assertTrue(show_body['router']['admin_state_up'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('802c73c9-c937-4cef-824b-2191e24a6aab')
    def test_add_multiple_router_interfaces(self):
        """Test adding multiple router interfaces"""
        network_name = data_utils.rand_name(self.__class__.__name__)
        network01 = self.networks_client.create_network(
            name=network_name)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network, network01['id'])
        network_name = data_utils.rand_name(self.__class__.__name__)
        network02 = self.networks_client.create_network(
            name=network_name)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network, network02['id'])
        subnet01 = self.create_subnet(network01)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnets_client.delete_subnet, subnet01['id'])
        sub02_cidr = self.cidr.next()
        subnet02 = self.create_subnet(network02, cidr=sub02_cidr)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnets_client.delete_subnet, subnet02['id'])
        router = self.create_router()
        self.addCleanup(self.delete_router, router)
        interface01 = self._add_router_interface_with_subnet_id(router['id'],
                                                                subnet01['id'])
        self._verify_router_interface(router['id'], subnet01['id'],
                                      interface01['port_id'])
        interface02 = self._add_router_interface_with_subnet_id(router['id'],
                                                                subnet02['id'])
        self._verify_router_interface(router['id'], subnet02['id'],
                                      interface02['port_id'])

    @decorators.idempotent_id('96522edf-b4b5-45d9-8443-fa11c26e6eff')
    def test_router_interface_port_update_with_fixed_ip(self):
        """Test updating router interface port's fixed ip"""
        network_name = data_utils.rand_name(self.__class__.__name__)
        network = self.networks_client.create_network(
            name=network_name)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network, network['id'])
        subnet = self.create_subnet(network)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnets_client.delete_subnet, subnet['id'])
        router = self.create_router()
        self.addCleanup(self.delete_router, router)
        fixed_ip = [{'subnet_id': subnet['id']}]
        interface = self._add_router_interface_with_subnet_id(router['id'],
                                                              subnet['id'])
        self.assertIn('port_id', interface)
        self.assertIn('subnet_id', interface)
        port = self.ports_client.show_port(interface['port_id'])
        self.assertEqual(port['port']['id'], interface['port_id'])
        router_port = self.ports_client.update_port(port['port']['id'],
                                                    fixed_ips=fixed_ip)
        self.assertEqual(subnet['id'],
                         router_port['port']['fixed_ips'][0]['subnet_id'])

    def _verify_router_interface(self, router_id, subnet_id, port_id):
        show_port_body = self.ports_client.show_port(port_id)
        interface_port = show_port_body['port']
        self.assertEqual(router_id, interface_port['device_id'])
        self.assertEqual(subnet_id,
                         interface_port['fixed_ips'][0]['subnet_id'])


class RoutersIpV6Test(RoutersTest):
    _ip_version = 6
